# Copyright 2020 Cloudification GmbH. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_log import log as logging
from oslo_utils import uuidutils
from oslo_utils import versionutils
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from designate_tempest_plugin.tests import base
from designate_tempest_plugin import data_utils as dns_data_utils

LOG = logging.getLogger(__name__)

CONF = config.CONF


class BaseSharedZoneTest(base.BaseDnsV2Test):

    credentials = ['admin', 'primary', 'alt', 'project_reader',
                   'project_member', ['demo', 'member']]

    excluded_keys = ['links']

    @classmethod
    def resource_setup(cls):
        super(BaseSharedZoneTest, cls).resource_setup()

        if not versionutils.is_compatible('2.1', cls.api_version,
                                          same_major=False):
            raise cls.skipException(
                'The shared zones API tests require Designate API version '
                '2.1 or newer. Skipping Shared Zones API tests.')

        # Make sure we have an allowed TLD available
        tld_name = dns_data_utils.rand_zone_name(name="APISharedZoneTest")
        cls.tld_name = f".{tld_name}"
        cls.class_tld = cls.admin_tld_client.create_tld(tld_name=tld_name[:-1])

        # All the shared zone tests need a zone, create one to share
        zone_name = dns_data_utils.rand_zone_name(name="TestZone",
                                                  suffix=cls.tld_name)
        LOG.info('Create a zone: %s', zone_name)
        cls.zone = cls.zones_client.create_zone(name=zone_name)[1]

    @classmethod
    def resource_cleanup(cls):
        cls.zones_client.delete_zone(
            cls.zone['id'], ignore_errors=lib_exc.NotFound, delete_shares=True)
        cls.admin_tld_client.delete_tld(cls.class_tld[1]['id'])
        super(BaseSharedZoneTest, cls).resource_cleanup()

    @classmethod
    def setup_clients(cls):
        super(BaseSharedZoneTest, cls).setup_clients()

        cls.admin_tld_client = cls.os_admin.dns_v2.TldClient()
        cls.adm_shr_client = cls.os_admin.dns_v2.SharedZonesClient()
        cls.alt_zone_client = cls.os_alt.dns_v2.ZonesClient()
        cls.demo_zone_client = cls.os_demo.dns_v2.ZonesClient()
        cls.share_zone_client = cls.os_primary.dns_v2.SharedZonesClient()
        cls.alt_share_zone_client = cls.os_alt.dns_v2.SharedZonesClient()


class SharedZonesTest(BaseSharedZoneTest):

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(SharedZonesTest, cls).setup_credentials()

    @decorators.idempotent_id('982a7780-a460-4c13-97df-b4855bf19c7b')
    def test_create_zone_share(self):
        # Test RBAC
        expected_allowed = ['os_admin', 'os_primary', 'os_alt']
        if CONF.dns_feature_enabled.enforce_new_defaults:
            expected_allowed.append('os_project_member')
        self.check_CUD_RBAC_enforcement(
            'SharedZonesClient', 'create_zone_share', expected_allowed, True,
            self.zone['id'], self.alt_zone_client.project_id)

        # Test a basic API create a zone share
        shared_zone = self.share_zone_client.create_zone_share(
            self.zone['id'], self.alt_zone_client.project_id)[1]
        self.addCleanup(self.share_zone_client.delete_zone_share,
                        self.zone['id'], shared_zone['id'])

        self.assertTrue(uuidutils.is_uuid_like(shared_zone['id']))
        self.assertEqual(self.zone['id'], shared_zone['zone_id'])
        self.assertEqual(self.share_zone_client.project_id,
                         shared_zone['project_id'])
        self.assertEqual(self.alt_zone_client.project_id,
                         shared_zone['target_project_id'])
        self.assertIsNotNone(shared_zone['created_at'])
        self.assertIsNone(shared_zone['updated_at'])
        self.assertIsNotNone(shared_zone['links'])

    @decorators.idempotent_id('0edecb9b-4890-433c-8195-0935271efc9a')
    def test_show_shared_zone(self):
        shared_zone = self.share_zone_client.create_zone_share(
            self.zone['id'], self.alt_zone_client.project_id)[1]
        self.addCleanup(self.share_zone_client.delete_zone_share,
                        self.zone['id'], shared_zone['id'])

        # Test RBAC
        expected_allowed = ['os_admin', 'os_primary', 'os_alt']
        if CONF.dns_feature_enabled.enforce_new_defaults:
            expected_allowed.append('os_project_member')
            expected_allowed.append('os_project_reader')
        self.check_CUD_RBAC_enforcement(
            'SharedZonesClient', 'show_zone_share', expected_allowed, True,
            self.zone['id'], shared_zone['id'])

        # Test show zone share
        LOG.info('Fetch the zone share')
        body = self.share_zone_client.show_zone_share(self.zone['id'],
                                                      shared_zone['id'])[1]

        LOG.info('Ensure the fetched response matches the zone share')
        self.assertExpected(shared_zone, body, self.excluded_keys)

    @decorators.idempotent_id('a18a8577-9d02-492a-a869-4ff7d6f4f89b')
    def test_delete_zone_share(self):
        shared_zone = self.share_zone_client.create_zone_share(
            self.zone['id'], self.alt_zone_client.project_id)[1]
        self.addCleanup(self.share_zone_client.delete_zone_share,
                        self.zone['id'], shared_zone['id'],
                        ignore_errors=lib_exc.NotFound)

        # Test RBAC
        expected_allowed = ['os_admin', 'os_primary']
        if CONF.dns_feature_enabled.enforce_new_defaults:
            expected_allowed.append('os_project_member')
        self.check_CUD_RBAC_enforcement(
            'SharedZonesClient', 'delete_zone_share', expected_allowed, True,
            self.zone['id'], shared_zone['id'])

        # Test zone share delete
        LOG.info('Delete zone share')
        self.share_zone_client.delete_zone_share(self.zone['id'],
                                                 shared_zone['id'])

        LOG.info('Ensure the zone share was deleted')
        self.assertRaises(lib_exc.NotFound,
            self.share_zone_client.show_zone_share,
            self.zone['id'], shared_zone['id'])

    @decorators.idempotent_id('707bfa4f-f15b-4486-ba5c-0e5991f0f3a5')
    def test_list_zone_shares(self):
        shared_zone = self.share_zone_client.create_zone_share(
            self.zone['id'], self.alt_zone_client.project_id)[1]
        self.addCleanup(self.share_zone_client.delete_zone_share,
                        self.zone['id'], shared_zone['id'])

        # Test RBAC
        expected_allowed = ['os_admin', 'os_primary', 'os_alt']
        if CONF.dns_feature_enabled.enforce_new_defaults:
            expected_allowed.append('os_project_member')
            expected_allowed.append('os_project_reader')
        self.check_CUD_RBAC_enforcement(
            'SharedZonesClient', 'list_zone_shares', expected_allowed, True,
            self.zone['id'])

        shared_zone = self.share_zone_client.create_zone_share(
            self.zone['id'], self.demo_zone_client.project_id)[1]
        self.addCleanup(self.share_zone_client.delete_zone_share,
                        self.zone['id'], shared_zone['id'])

        LOG.info('List zone shares')
        body = self.share_zone_client.list_zone_shares(self.zone['id'])[1]

        self.assertEqual(2, len(body['shared_zones']))
        targets = []
        for share in body['shared_zones']:
            targets.append(share['target_project_id'])
        self.assertIn(self.alt_zone_client.project_id, targets)
        self.assertIn(self.demo_zone_client.project_id, targets)


class NegativeSharedZonesTest(BaseSharedZoneTest):

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(NegativeSharedZonesTest, cls).setup_credentials()

    @decorators.idempotent_id('4389a12b-8609-493c-9640-d3c67b625022')
    def test_target_project_cannot_delete_zone(self):
        shared_zone = self.share_zone_client.create_zone_share(
            self.zone['id'], self.alt_zone_client.project_id)[1]
        self.addCleanup(self.share_zone_client.delete_zone_share,
                        self.zone['id'], shared_zone['id'])

        LOG.info('Ensure target project cannot delete zone')
        self.assertRaises(lib_exc.Forbidden,
                          self.alt_zone_client.delete_zone,
                          self.zone['id'])

    @decorators.idempotent_id('f4354b5c-8dbb-4bb9-8025-f65f8f2b21fb')
    def test_target_project_cannot_update_zone(self):
        shared_zone = self.share_zone_client.create_zone_share(
            self.zone['id'], self.alt_zone_client.project_id)[1]
        self.addCleanup(self.share_zone_client.delete_zone_share,
                        self.zone['id'], shared_zone['id'])

        LOG.info('Ensure target project cannot update the zone')
        self.assertRaises(lib_exc.Forbidden,
                          self.alt_zone_client.update_zone,
                          self.zone['id'], ttl=5)

    @decorators.idempotent_id('4389a12b-8609-493c-9640-d3c67b625022')
    def test_target_project_share_permissions(self):
        shared_zone = self.share_zone_client.create_zone_share(
            self.zone['id'], self.alt_zone_client.project_id)[1]
        self.addCleanup(self.share_zone_client.delete_zone_share,
                        self.zone['id'], shared_zone['id'])

        LOG.info('Ensure target project cannot share shared zone')
        self.assertRaises(
            lib_exc.Forbidden,
            self.alt_share_zone_client.create_zone_share,
            self.zone['id'],
            self.demo_zone_client.project_id)

    @decorators.idempotent_id('abc0f820-ae27-4e85-8f00-0b8e8abf3ae9')
    def test_target_project_cannot_subzone(self):
        shared_zone = self.share_zone_client.create_zone_share(
            self.zone['id'], self.alt_zone_client.project_id)[1]
        self.addCleanup(self.share_zone_client.delete_zone_share,
                        self.zone['id'], shared_zone['id'])

        LOG.info('Ensure target project cannot create sub-zones')
        sub_zone_name = "test.{}".format(self.zone['name'])
        self.assertRaises(
            lib_exc.Forbidden,
            self.alt_zone_client.create_zone,
            name=sub_zone_name)

    @decorators.idempotent_id('957ba3f8-c250-11ed-a8b1-201e8823901f')
    def test_share_zone_with_yourself_is_not_allowed(self):
        with self.assertRaisesDns(lib_exc.BadRequest, 'bad_request', 400):
            self.share_zone_client.create_zone_share(
                zone_id=self.zone['id'],
                target_project_id=self.share_zone_client.project_id)


class AdminSharedZonesTest(BaseSharedZoneTest):

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(AdminSharedZonesTest, cls).setup_credentials()

    @decorators.idempotent_id('2bb7bcb2-b824-11ed-9e56-201e8823901f')
    def test_create_zone_share_all_projects_header(self):
        LOG.info(
            'Admin user creates shared zone for Alt tenant '
            'using "x-auth-all-projects" header')
        # Scoped tokens do not have a project ID, work around that here
        if CONF.enforce_scope.designate:
            headers = self.all_projects_header.copy()
            headers.update(
                {'x-auth-sudo-project-id': self.share_zone_client.project_id})
        else:
            headers = self.all_projects_header

        shared_zone = self.adm_shr_client.create_zone_share(
            self.zone['id'], self.alt_zone_client.project_id,
            headers=headers)[1]
        self.addCleanup(
            self.adm_shr_client.delete_zone_share, self.zone['id'],
            shared_zone['id'], headers=self.all_projects_header)
        self.assertTrue(uuidutils.is_uuid_like(shared_zone['id']))
        self.assertEqual(self.zone['id'], shared_zone['zone_id'])
        if CONF.enforce_scope.designate:
            self.assertEqual(self.share_zone_client.project_id,
                             shared_zone['project_id'])
        else:
            self.assertEqual(self.adm_shr_client.project_id,
                             shared_zone['project_id'])
        self.assertEqual(self.alt_zone_client.project_id,
                         shared_zone['target_project_id'])
        self.assertIsNotNone(shared_zone['created_at'])
        self.assertIsNone(shared_zone['updated_at'])
        self.assertIsNotNone(shared_zone['links'])

    @decorators.idempotent_id('f26cd3ac-b8fa-11ed-b4ca-201e8823901f')
    def test_create_zone_share_sudo_project_header(self):
        LOG.info(
            'Admin user creates shared zone for Alt tenant '
            'using "x-auth-sudo-project-id" header')
        sudo_header = {
            'x-auth-sudo-project-id': self.share_zone_client.project_id}
        shared_zone = self.adm_shr_client.create_zone_share(
            self.zone['id'], self.alt_zone_client.project_id,
            headers=sudo_header)[1]
        self.addCleanup(
            self.adm_shr_client.delete_zone_share, self.zone['id'],
            shared_zone['id'], headers=sudo_header)
        self.assertTrue(uuidutils.is_uuid_like(shared_zone['id']))
        self.assertEqual(self.zone['id'], shared_zone['zone_id'])
        self.assertEqual(self.share_zone_client.project_id,
                         shared_zone['project_id'])
        self.assertEqual(self.alt_zone_client.project_id,
                         shared_zone['target_project_id'])
        self.assertIsNotNone(shared_zone['created_at'])
        self.assertIsNone(shared_zone['updated_at'])
        self.assertIsNotNone(shared_zone['links'])

    @decorators.idempotent_id('ce2688e8-b90a-11ed-b4ca-201e8823901f')
    def test_show_shared_zone_all_projects_header(self):
        LOG.info(
            'Admin user creates shared zone for Alt tenant'
            ' using "x-auth-all-projects" header')
        # Scoped tokens do not have a project ID, work around that here
        if CONF.enforce_scope.designate:
            headers = self.all_projects_header.copy()
            headers.update(
                {'x-auth-sudo-project-id': self.share_zone_client.project_id})
        else:
            headers = self.all_projects_header

        shared_zone = self.adm_shr_client.create_zone_share(
            self.zone['id'], self.alt_zone_client.project_id,
            headers=headers)[1]
        self.addCleanup(
            self.adm_shr_client.delete_zone_share, self.zone['id'],
            shared_zone['id'], headers=self.all_projects_header)

        LOG.info('Admin user shows shared zone and validates its content')
        body = self.adm_shr_client.show_zone_share(
            self.zone['id'], shared_zone['id'],
            headers=self.all_projects_header)[1]
        self.assertExpected(shared_zone, body, self.excluded_keys)

    @decorators.idempotent_id('46f7db22-b90c-11ed-b4ca-201e8823901f')
    def test_delete_zone_share_sudo_project_header(self):
        LOG.info(
            'Admin user creates shared zone for Alt tenant'
            ' using "x-auth-sudo-project-id" header')
        sudo_header = {
            'x-auth-sudo-project-id': self.share_zone_client.project_id}
        shared_zone = self.adm_shr_client.create_zone_share(
            self.zone['id'], self.alt_zone_client.project_id,
            headers=sudo_header)[1]
        self.addCleanup(
            self.adm_shr_client.delete_zone_share, self.zone['id'],
            shared_zone['id'], headers=sudo_header,
            ignore_errors=lib_exc.NotFound)

        LOG.info('As Admin delete zone share and ensure it was deleted')
        self.adm_shr_client.delete_zone_share(
            self.zone['id'], shared_zone['id'], headers=sudo_header)
        self.assertRaises(lib_exc.NotFound,
            self.adm_shr_client.show_zone_share,
            self.zone['id'], shared_zone['id'], headers=sudo_header)

    @decorators.idempotent_id('2eedfd60-b90f-11ed-b4ca-201e8823901f')
    def test_list_zone_shares_all_projects_header(self):
        LOG.info(
            "Admin user shares Primary's zone with Alt tenant"
            " using 'x-auth-all-projects' header")
        # Scoped tokens do not have a project ID, work around that here
        if CONF.enforce_scope.designate:
            headers = self.all_projects_header.copy()
            headers.update(
                {'x-auth-sudo-project-id': self.share_zone_client.project_id})
        else:
            headers = self.all_projects_header

        shared_zone = self.adm_shr_client.create_zone_share(
            self.zone['id'], self.alt_zone_client.project_id,
            headers=headers)[1]
        self.addCleanup(
            self.adm_shr_client.delete_zone_share, self.zone['id'],
            shared_zone['id'], headers=self.all_projects_header)

        LOG.info(
            "Admin user shares Primary's zone with Demo tenant"
            " using 'x-auth-all-projects' header")
        shared_zone = self.adm_shr_client.create_zone_share(
            self.zone['id'], self.demo_zone_client.project_id,
            headers=headers)[1]
        self.addCleanup(
            self.adm_shr_client.delete_zone_share, self.zone['id'],
            shared_zone['id'], headers=self.all_projects_header)

        LOG.info('Admin user lists zone shares')
        body = self.adm_shr_client.list_zone_shares(
            self.zone['id'], headers=self.all_projects_header)[1]

        self.assertEqual(2, len(body['shared_zones']))
        targets = []
        for share in body['shared_zones']:
            targets.append(share['target_project_id'])
        self.assertIn(self.alt_zone_client.project_id, targets)
        self.assertIn(self.demo_zone_client.project_id, targets)


class AdminSharedZonesTestNegative(BaseSharedZoneTest):

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(AdminSharedZonesTestNegative, cls).setup_credentials()

    @decorators.idempotent_id('595ae1fc-bce4-11ed-baf2-201e8823901f')
    def test_create_zone_share_invalid_project_id(self):
        LOG.info(
            'Admin user tries to create shared zone, using non existing '
            'project ID id in "x-auth-sudo-project-id" header')
        sudo_header = {
            'x-auth-sudo-project-id': data_utils.rand_uuid()}
        self.assertRaises(
            lib_exc.NotFound, self.adm_shr_client.create_zone_share,
            self.zone['id'], self.alt_zone_client.project_id,
            headers=sudo_header)

    @decorators.idempotent_id('aa42d82e-bcf6-11ed-baf2-201e8823901f')
    def test_create_zone_share_invalid_zone_id(self):
        LOG.info(
            'Admin user tries to create shared zone, using non existing '
            'zone ID and "x-auth-sudo-project-id" header')
        sudo_header = {
            'x-auth-sudo-project-id': self.alt_zone_client.project_id}
        self.assertRaises(
            lib_exc.NotFound, self.adm_shr_client.create_zone_share,
            data_utils.rand_uuid(), self.alt_zone_client.project_id,
            headers=sudo_header)

    @decorators.idempotent_id('9e7202ba-bd94-11ed-80f5-201e8823901f')
    def test_show_shared_zone_invalid_shared_zone_id(self):
        LOG.info('Admin tries to show shared zone using not '
                 'existing shared zone ID')
        sudo_header = {
            'x-auth-sudo-project-id': self.alt_zone_client.project_id}
        self.assertRaises(
            lib_exc.NotFound, self.adm_shr_client.show_zone_share,
            self.zone['id'], data_utils.rand_uuid(), headers=sudo_header)

    @decorators.idempotent_id('8852329c-bd95-11ed-80f5-201e8823901f')
    def test_show_shared_zone_invalid_project_id(self):
        LOG.info('Admin tries to show shared zone '
                 'using not existing project ID')
        sudo_header = {
            'x-auth-sudo-project-id': self.share_zone_client.project_id}
        shared_zone = self.adm_shr_client.create_zone_share(
            self.zone['id'], self.alt_zone_client.project_id,
            headers=sudo_header)[1]
        self.addCleanup(
            self.adm_shr_client.delete_zone_share, self.zone['id'],
            shared_zone['id'], headers=sudo_header)
        sudo_header_invalid_project_id = {
            'x-auth-sudo-project-id': data_utils.rand_uuid()}
        self.assertRaises(
            lib_exc.NotFound, self.adm_shr_client.show_zone_share,
            self.zone['id'], shared_zone['id'],
            headers=sudo_header_invalid_project_id)

    @decorators.idempotent_id('871e7e1c-bd9a-11ed-80f5-201e8823901f')
    @decorators.skip_because(bug="2009819")
    def test_list_zone_shares_invalid_zone_id(self):
        LOG.info('Admin user tries to list shared zone '
                 'using not existing zone ID')
        sudo_header = {
            'x-auth-sudo-project-id': self.share_zone_client.project_id}
        self.assertRaises(
            lib_exc.NotFound, self.adm_shr_client.list_zone_shares,
            data_utils.rand_uuid(), headers=sudo_header)

    @decorators.idempotent_id('e71068c8-bdb1-11ed-80f5-201e8823901f')
    @ decorators.skip_because(bug="2009819")
    def test_list_zone_shares_invalid_project_id(self):
        LOG.info('Admin user tries to list shared zone using '
                 'not existing project ID')
        sudo_header = {
            'x-auth-sudo-project-id': data_utils.rand_uuid()}
        self.assertRaises(
            lib_exc.NotFound, self.adm_shr_client.list_zone_shares,
            self.zone['id'], headers=sudo_header)

    @decorators.idempotent_id('7136b430-bdb2-11ed-80f5-201e8823901f')
    def test_delete_zone_share_invalid_project_id(self):
        LOG.info('Admin user creates shared zone for Alt user')
        sudo_header = {
            'x-auth-sudo-project-id': self.share_zone_client.project_id}
        shared_zone = self.adm_shr_client.create_zone_share(
            self.zone['id'], self.alt_zone_client.project_id,
            headers=sudo_header)[1]
        self.addCleanup(
            self.adm_shr_client.delete_zone_share, self.zone['id'],
            shared_zone['id'], headers=sudo_header,
            ignore_errors=lib_exc.NotFound)
        LOG.info('Admin user tries to delete the shared zone '
                 'using non existing project ID')
        invalid_sudo_header = {
            'x-auth-sudo-project-id': data_utils.rand_uuid()}
        self.assertRaises(
            lib_exc.NotFound, self.adm_shr_client.delete_zone_share,
            self.zone['id'], shared_zone['id'],
            headers=invalid_sudo_header)

    @decorators.idempotent_id('d44c65e2-bdc1-11ed-80f5-201e8823901f')
    def test_delete_zone_share_invalid_shared_zone_id(self):
        LOG.info('Admin user creates shared zone for Alt user')
        sudo_header = {
            'x-auth-sudo-project-id': self.share_zone_client.project_id}
        shared_zone = self.adm_shr_client.create_zone_share(
            self.zone['id'], self.alt_zone_client.project_id,
            headers=sudo_header)[1]
        self.addCleanup(
            self.adm_shr_client.delete_zone_share, self.zone['id'],
            shared_zone['id'], headers=sudo_header,
            ignore_errors=lib_exc.NotFound)
        LOG.info('Admin user tries to delete the shared zone '
                 'using non existing shared zone ID')
        self.assertRaises(
            lib_exc.NotFound, self.adm_shr_client.delete_zone_share,
            self.zone['id'], data_utils.rand_uuid(),
            headers=sudo_header)

    @decorators.idempotent_id('06de2342-bdc2-11ed-80f5-201e8823901f')
    def test_delete_zone_share_invalid_zone_id(self):
        LOG.info('Admin user creates shared zone for Alt user')
        sudo_header = {
            'x-auth-sudo-project-id': self.share_zone_client.project_id}
        shared_zone = self.adm_shr_client.create_zone_share(
            self.zone['id'], self.alt_zone_client.project_id,
            headers=sudo_header)[1]
        self.addCleanup(
            self.adm_shr_client.delete_zone_share, self.zone['id'],
            shared_zone['id'], headers=sudo_header,
            ignore_errors=lib_exc.NotFound)
        LOG.info('Admin user tries to delete the shared zone '
                 'using non zone ID')
        self.assertRaises(
            lib_exc.NotFound, self.adm_shr_client.delete_zone_share,
            data_utils.rand_uuid(), shared_zone['id'],
            headers=sudo_header)



# Copyright 2021 Red Hat.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_log import log as logging
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from designate_tempest_plugin import data_utils as dns_data_utils
from designate_tempest_plugin.tests import base

CONF = config.CONF
LOG = logging.getLogger(__name__)


class BaseBlacklistsTest(base.BaseDnsV2Test):
    excluded_keys = ['created_at', 'updated_at', 'links']

    @classmethod
    def setup_clients(cls):
        super(BaseBlacklistsTest, cls).setup_clients()

        cls.admin_tld_client = cls.os_admin.dns_v2.TldClient()

    @classmethod
    def resource_setup(cls):
        super(BaseBlacklistsTest, cls).resource_setup()

        # Make sure we have an allowed TLD available
        tld_name = dns_data_utils.rand_zone_name(name="BaseBlacklistsTest")
        cls.tld_name = f".{tld_name}"
        cls.class_tld = cls.admin_tld_client.create_tld(tld_name=tld_name[:-1])

    @classmethod
    def resource_cleanup(cls):
        cls.admin_tld_client.delete_tld(cls.class_tld[1]['id'])
        super(BaseBlacklistsTest, cls).resource_cleanup()


class BlacklistE2E(BaseBlacklistsTest):

    credentials = ["admin", 'primary']

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(BlacklistE2E, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(BlacklistE2E, cls).setup_clients()
        cls.admin_blacklist_client = cls.os_admin.dns_v2.BlacklistsClient()
        cls.admin_zone_client = cls.os_admin.dns_v2.ZonesClient()

    @decorators.idempotent_id('22b1ee72-d8d2-11eb-bcdc-74e5f9e2a801')
    def test_primary_fails_to_create_zone_matches_blacklist_regex(self):
        LOG.info('Create a blacklist using regex')
        blacklist = {
            'pattern': '^blacklistregextest.*',
            'description': 'Zone starts with "blacklistregextest" char'}
        body = self.admin_blacklist_client.create_blacklist(**blacklist)[1]
        self.addCleanup(
            self.admin_blacklist_client.delete_blacklist, body['id'])

        LOG.info('Try to create a zone that is starts with '
                 '"blacklistregextest".')
        self.assertRaisesDns(
            lib_exc.BadRequest, 'invalid_zone_name', 400,
            self.zones_client.create_zone,
            name='blacklistregextest' + dns_data_utils.rand_zone_name())

    @decorators.idempotent_id('6956f20c-d8d5-11eb-bcdc-74e5f9e2a801')
    def test_primary_fails_to_create_zone_matches_blacklist_name(self):
        LOG.info('Create a blacklist using the exact name(string)')
        zone_name = 'blacklistnametest' + dns_data_utils.rand_zone_name()
        blacklist = {
            'pattern': zone_name,
            'description': 'Zone named:{} '.format(zone_name)}
        body = self.admin_blacklist_client.create_blacklist(**blacklist)[1]
        self.addCleanup(
            self.admin_blacklist_client.delete_blacklist, body['id'])

        LOG.info('Try to create a zone named:{}'.format(zone_name))
        self.assertRaisesDns(
            lib_exc.BadRequest, 'invalid_zone_name', 400,
            self.zones_client.create_zone, name=zone_name)

    @decorators.idempotent_id('de030088-d97e-11eb-8ab8-74e5f9e2a801')
    def test_admin_creates_zone_matches_blacklist_name_or_regex(self):
        LOG.info('Create two blacklists: by regex and by exact string')
        zone_name_to_deny = dns_data_utils.rand_zone_name(
            name="deny_by_name", suffix=self.tld_name)
        blacklists = [
            {'pattern': '^blacklistnameregextest2.*',
             'description': 'Zone starts with "blacklistnameregextest2"'},
            {'pattern': zone_name_to_deny,
             'description': 'Deny if Zone named:{} '.format(
                 zone_name_to_deny)}]
        for blacklist in blacklists:
            body = self.admin_blacklist_client.create_blacklist(**blacklist)[1]
            self.addCleanup(
                self.admin_blacklist_client.delete_blacklist, body['id'])

        LOG.info('Primary tries to create a zone that is blacklisted by name.'
                 ' Expected: FAIL')
        with self.assertRaisesDns(
                lib_exc.BadRequest, 'invalid_zone_name', 400):
            self.zones_client.create_zone(name=zone_name_to_deny)

        LOG.info('Admin tries to create a zone that is blacklisted by name '
                 'for a Primary user. Expected: FAIL')
        zone = self.admin_zone_client.create_zone(
            name=zone_name_to_deny,
            project_id=self.zones_client.project_id)[1]
        self.addCleanup(
            self.wait_zone_delete, self.zones_client, zone['id'])

        LOG.info('Primary tries to create a zone that is blacklisted by regex.'
                 ' Expected: FAIL')
        with self.assertRaisesDns(
                lib_exc.BadRequest, 'invalid_zone_name', 400):
            self.zones_client.create_zone(
                name='blacklistnameregextest2{}'.format(zone_name_to_deny))

        LOG.info('Admin tries to create a zone that is blacklisted by regex'
                 ' for a Primary user. Expected: FAIL')
        zone = self.admin_zone_client.create_zone(
            name='blacklistnameregextest2{}'.format(zone_name_to_deny),
            project_id=self.zones_client.project_id)[1]
        self.addCleanup(
            self.wait_zone_delete, self.zones_client, zone['id'])



# Copyright 2016 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from operator import itemgetter

import testtools
from oslo_log import log as logging
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from tempest.lib.common.utils import data_utils

from designate_tempest_plugin.tests import base

CONF = config.CONF
LOG = logging.getLogger(__name__)


class BasePoolTest(base.BaseDnsV2Test):
    excluded_keys = ['created_at', 'updated_at', 'links', 'ns_records']

    def _assertExpectedNSRecords(self, expected, actual, expected_key):
        sort_expected = sorted(expected, key=itemgetter(expected_key))
        sort_actual = sorted(actual, key=itemgetter(expected_key))
        self.assertEqual(sort_expected, sort_actual)


class PoolAdminTest(BasePoolTest):
    credentials = ["admin", "primary", "project_member",
                   "project_reader", "alt"]

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(PoolAdminTest, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(PoolAdminTest, cls).setup_clients()
        cls.admin_client = cls.os_admin.dns_v2.PoolClient()

    @decorators.idempotent_id('69257f7c-b3d5-4e1b-998e-0677ad12f125')
    @testtools.skipIf(CONF.dns_feature_enabled.test_multipool_with_delete_opt,
                      'Multipools feature is being tested with --delete '
                      'option. It might delete pools that were created in '
                      'other tests.')
    def test_create_pool(self):
        pool_data = {
                      "name": "Example Pool",
                      "project_id": "1",
                      "ns_records": [{
                          "hostname": "ns1.example.org.",
                          "priority": 1}
                      ]
                    }
        LOG.info('Create a pool')
        _, pool = self.admin_client.create_pool(pool_name=pool_data["name"],
            ns_records=pool_data["ns_records"],
            project_id=pool_data["project_id"])
        self.addCleanup(self.admin_client.delete_pool, pool['id'],
                        headers=self.all_projects_header)

        self.assertEqual(pool_data["name"], pool['name'])
        self.assertExpected(pool_data, pool, self.excluded_keys)

        # Test RBAC
        expected_allowed = ['os_admin']

        self.check_CUD_RBAC_enforcement(
            'PoolClient', 'create_pool', expected_allowed, False,
            pool_name=pool_data["name"], ns_records=pool_data["ns_records"],
            project_id=pool_data["project_id"])

    @decorators.idempotent_id('e80eb70a-8ee5-40eb-b06e-599597a8ab7e')
    @testtools.skipIf(CONF.dns_feature_enabled.test_multipool_with_delete_opt,
                      'Multipools feature is being tested with --delete '
                      'option. It might delete pools that were created in '
                      'other tests.')
    def test_show_pool(self):
        LOG.info('Create a pool')
        _, pool = self.admin_client.create_pool(project_id="1")
        self.addCleanup(self.admin_client.delete_pool, pool['id'],
                        headers=self.all_projects_header)

        LOG.info('Fetch the pool')
        _, body = self.admin_client.show_pool(
            pool['id'], headers=self.all_projects_header)

        LOG.info('Ensure the fetched response matches the created pool')
        self.assertExpected(pool, body, self.excluded_keys)
        self._assertExpectedNSRecords(pool["ns_records"], body["ns_records"],
                                expected_key="priority")

        # TODO(johnsom) Test reader roles once this bug is fixed.
        #               https://bugs.launchpad.net/tempest/+bug/1964509
        # Test RBAC
        expected_allowed = ['os_admin']

        # TODO(johnsom) The pools API seems inconsistent with the requirement
        #               of the all-projects header.
        self.check_list_show_RBAC_enforcement(
            'PoolClient', 'show_pool', expected_allowed, True, pool['id'],
            headers=self.all_projects_header)

    @decorators.idempotent_id('d8c4c377-5d88-452d-a4d2-c004d72e1abe')
    @testtools.skipIf(CONF.dns_feature_enabled.test_multipool_with_delete_opt,
                      'Multipools feature is being tested with --delete '
                      'option. It might delete pools that were created in '
                      'other tests.')
    def test_delete_pool(self):
        LOG.info('Create a pool')
        _, pool = self.admin_client.create_pool(project_id="1")
        self.addCleanup(self.admin_client.delete_pool, pool['id'],
                        ignore_errors=lib_exc.NotFound,
                        headers=self.all_projects_header)

        LOG.info('Delete the pool')
        _, body = self.admin_client.delete_pool(
            pool['id'], headers=self.all_projects_header)

        self.assertRaises(lib_exc.NotFound,
           lambda: self.admin_client.show_pool(
               pool['id'], headers=self.all_projects_header))

        # Test RBAC
        expected_allowed = ['os_admin']

        self.check_CUD_RBAC_enforcement(
            'PoolClient', 'delete_pool', expected_allowed, False, pool['id'])

    @decorators.idempotent_id('77c85b40-83b2-4c17-9fbf-e6d516cfce90')
    @testtools.skipIf(CONF.dns_feature_enabled.test_multipool_with_delete_opt,
                      'Multipools feature is being tested with --delete '
                      'option. It might delete pools that were created in '
                      'other tests.')
    def test_list_pools(self):
        LOG.info('Create a pool')
        _, pool = self.admin_client.create_pool(project_id="1")
        self.addCleanup(self.admin_client.delete_pool, pool['id'],
                        headers=self.all_projects_header)

        LOG.info('List pools')
        _, body = self.admin_client.list_pools(
            headers=self.all_projects_header)

        self.assertGreater(len(body['pools']), 0)

        # TODO(johnsom) Test reader roles once this bug is fixed.
        #               https://bugs.launchpad.net/tempest/+bug/1964509
        # Test RBAC
        expected_allowed = ['os_admin']

        self.check_list_IDs_RBAC_enforcement(
            'PoolClient', 'list_pools', expected_allowed, [pool['id']],
            headers=self.all_projects_header)

    @decorators.idempotent_id('fdcc84ce-af65-4af6-a5fc-6c50acbea0f0')
    @testtools.skipIf(CONF.dns_feature_enabled.test_multipool_with_delete_opt,
                      'Multipools feature is being tested with --delete '
                      'option. It might delete pools that were created in '
                      'other tests.')
    def test_update_pool(self):
        LOG.info('Create a pool')
        _, pool = self.admin_client.create_pool(project_id="1")
        self.addCleanup(self.admin_client.delete_pool, pool['id'],
                        headers=self.all_projects_header)

        LOG.info('Update the pool')
        _, patch_pool = self.admin_client.update_pool(
            pool['id'], pool_name="foo", headers=self.all_projects_header,
            extra_headers=True)

        self.assertEqual("foo", patch_pool["name"])

        # Test RBAC
        expected_allowed = ['os_admin']

        self.check_CUD_RBAC_enforcement(
            'PoolClient', 'update_pool', expected_allowed, True,
            pool['id'], pool_name="test-name")

    @decorators.idempotent_id('41ad6a84-00ce-4a04-9fd5-b7c15c31e2db')
    def test_list_pools_dot_json_fails(self):
        uri = self.admin_client.get_uri('pools.json')

        self.assertRaises(lib_exc.NotFound,
            lambda: self.admin_client.get(uri))


class TestPoolNotFoundAdmin(BasePoolTest):

    credentials = ["admin", "primary"]

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(TestPoolNotFoundAdmin, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(TestPoolNotFoundAdmin, cls).setup_clients()
        cls.admin_client = cls.os_admin.dns_v2.PoolClient()

    @decorators.idempotent_id('56281b2f-dd5a-4376-8c32-aba771062fa5')
    def test_show_pool_404(self):
        e = self.assertRaises(lib_exc.NotFound,
                              self.admin_client.show_pool,
                              data_utils.rand_uuid())
        self.assertPool404(e.resp, e.resp_body)

    @decorators.idempotent_id('10fba3c2-9972-479c-ace1-8f7eac7c159f')
    def test_update_pool_404(self):
        e = self.assertRaises(lib_exc.NotFound,
                              self.admin_client.update_pool,
                              data_utils.rand_uuid())
        self.assertPool404(e.resp, e.resp_body)

    @decorators.idempotent_id('96132295-896b-4de3-8f86-cc2ee513fdad')
    def test_delete_pool_404(self):
        e = self.assertRaises(lib_exc.NotFound,
                              self.admin_client.delete_pool,
                              data_utils.rand_uuid())
        self.assertPool404(e.resp, e.resp_body)

    def assertPool404(self, resp, resp_body):
        self.assertEqual(404, resp.status)
        self.assertEqual(404, resp_body['code'])
        self.assertEqual("pool_not_found", resp_body['type'])
        self.assertEqual("Could not find Pool", resp_body['message'])


class TestPoolInvalidIdAdmin(BasePoolTest):

    credentials = ["admin", "primary"]

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(TestPoolInvalidIdAdmin, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(TestPoolInvalidIdAdmin, cls).setup_clients()
        cls.admin_client = cls.os_admin.dns_v2.PoolClient()

    @decorators.idempotent_id('081d0188-42a7-4953-af0e-b022960715e2')
    def test_show_pool_invalid_uuid(self):
        e = self.assertRaises(lib_exc.BadRequest,
                              self.admin_client.show_pool,
                              'foo')
        self.assertPoolInvalidId(e.resp, e.resp_body)

    @decorators.idempotent_id('f4ab4f5a-d7f0-4758-b232-8338f02d7c5c')
    def test_update_pool_invalid_uuid(self):
        e = self.assertRaises(lib_exc.BadRequest,
                              self.admin_client.update_pool,
                              'foo')
        self.assertPoolInvalidId(e.resp, e.resp_body)

    @decorators.idempotent_id('bf5ad3be-2e79-439d-b247-902fe198143b')
    def test_delete_pool_invalid_uuid(self):
        e = self.assertRaises(lib_exc.BadRequest,
                              self.admin_client.delete_pool,
                              'foo')
        self.assertPoolInvalidId(e.resp, e.resp_body)

    def assertPoolInvalidId(self, resp, resp_body):
        self.assertEqual(400, resp.status)
        self.assertEqual(400, resp_body['code'])
        self.assertEqual("invalid_uuid", resp_body['type'])
        self.assertEqual("Invalid UUID pool_id: foo",
                         resp_body['message'])


class TestPoolAdminNegative(BasePoolTest):

    credentials = ["admin", "primary"]

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(TestPoolAdminNegative, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(TestPoolAdminNegative, cls).setup_clients()
        cls.admin_client = cls.os_admin.dns_v2.PoolClient()

    @decorators.idempotent_id('0a8cdc1e-ac02-11eb-ae06-74e5f9e2a801')
    def test_create_pool_invalid_name(self):
        LOG.info('Create a pool using a huge size string for name)')
        with self.assertRaisesDns(lib_exc.BadRequest, 'invalid_object', 400):
            self.admin_client.create_pool(
                pool_name=data_utils.rand_name(name="Huge_size_name") * 100)

    @decorators.idempotent_id('9a787d0e-ac04-11eb-ae06-74e5f9e2a801')
    def test_create_pool_invalid_hostname_in_ns_records(self):
        LOG.info('Create a pool using invalid hostname in ns_records')
        with self.assertRaisesDns(lib_exc.BadRequest, 'invalid_object', 400):
            self.admin_client.create_pool(
                ns_records=[{"hostname": "ns1_example_org_", "priority": 1}])

    @decorators.idempotent_id('9a787d0e-ac04-11eb-ae06-74e5f9e2a801')
    def test_create_pool_invalid_priority_in_ns_records(self):
        LOG.info('Create a pool using invalid priority in ns_records')
        with self.assertRaisesDns(lib_exc.BadRequest, 'invalid_object', 400):
            self.admin_client.create_pool(
                ns_records=[{"hostname": "ns1.example.org.", "priority": -1}])

    @decorators.idempotent_id('cc378e4c-ac05-11eb-ae06-74e5f9e2a801')
    @testtools.skipIf(CONF.dns_feature_enabled.test_multipool_with_delete_opt,
                      'Multipools feature is being tested with --delete '
                      'option. It might delete pools that were created in '
                      'other tests.')
    # Note: Update pool API is deprecated for removal.
    def test_update_pool_with_invalid_name(self):
        LOG.info('Create a pool')
        pool = self.admin_client.create_pool(project_id="1")[1]
        self.addCleanup(self.admin_client.delete_pool, pool['id'],
                        headers=self.all_projects_header)

        LOG.info('Update the pool using a name that is too long')
        with self.assertRaisesDns(lib_exc.BadRequest, 'invalid_object', 400):
            self.admin_client.update_pool(
                pool['id'],
                pool_name=data_utils.rand_name(name="Huge_size_name") * 100,
                headers=self.all_projects_header, extra_headers=True)

    @decorators.idempotent_id('2e496596-ac07-11eb-ae06-74e5f9e2a801')
    @testtools.skipIf(CONF.dns_feature_enabled.test_multipool_with_delete_opt,
                      'Multipools feature is being tested with --delete '
                      'option. It might delete pools that were created in '
                      'other tests.')
    def test_update_pool_with_invalid_hostname_in_ns_records(self):
        # Note: Update pool API is deprecated for removal.
        LOG.info('Create a pool')
        pool = self.admin_client.create_pool(project_id="1")[1]
        self.addCleanup(self.admin_client.delete_pool, pool['id'],
                        headers=self.all_projects_header)

        LOG.info('Update the pool using invalid hostname in ns_records')
        with self.assertRaisesDns(lib_exc.BadRequest, 'invalid_object', 400):
            self.admin_client.update_pool(
                pool['id'],
                ns_records=[{"hostname": "ns1_example_org_", "priority": 1}],
                headers=self.all_projects_header, extra_headers=True)

    @decorators.idempotent_id('3e934624-ac07-11eb-ae06-74e5f9e2a801')
    @testtools.skipIf(CONF.dns_feature_enabled.test_multipool_with_delete_opt,
                      'Multipools feature is being tested with --delete '
                      'option. It might delete pools that were created in '
                      'other tests.')
    def test_update_pool_with_invalid_priority_in_ns_records(self):
        # Note: Update pool API is deprecated for removal.
        LOG.info('Create a pool')
        pool = self.admin_client.create_pool(project_id="1")[1]
        self.addCleanup(self.admin_client.delete_pool, pool['id'],
                        headers=self.all_projects_header)

        LOG.info('Update the pool using invalid priority in ns_records')
        with self.assertRaisesDns(lib_exc.BadRequest, 'invalid_object', 400):
            self.admin_client.update_pool(
                pool['id'],
                ns_records=[{"hostname": "ns1.example.org.", "priority": -1}],
                headers=self.all_projects_header, extra_headers=True)



# Copyright 2016 Hewlett Packard Enterprise Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import time

from oslo_log import log as logging
from tempest.lib.common.utils import test_utils
from tempest.lib import exceptions as lib_exc

from designate_tempest_plugin.common import constants as const
from designate_tempest_plugin.common import exceptions

LOG = logging.getLogger(__name__)


def wait_for_zone_404(client, zone_id):
    """Waits for a zone to 404."""
    LOG.info('Waiting for zone %s to 404', zone_id)
    start = int(time.time())

    while True:
        time.sleep(client.build_interval)

        try:
            zone = client.show_zone(zone_id)[1]
        except lib_exc.NotFound:
            LOG.info('Zone %s is 404ing', zone_id)
            return

        if zone['status'] == const.ERROR:
            raise exceptions.InvalidStatusError('Zone', zone_id,
                                                zone['status'])

        if int(time.time()) - start >= client.build_timeout:
            message = ('Zone %(zone_id)s failed to 404 within the required '
                       'time (%(timeout)s s). Current status: '
                       '%(status_curr)s' %
                       {'zone_id': zone_id,
                        'status_curr': zone['status'],
                        'timeout': client.build_timeout})

            caller = test_utils.find_test_caller()

            if caller:
                message = '(%s) %s' % (caller, message)

            raise lib_exc.TimeoutException(message)


def wait_for_zone_status(client, zone_id, status, headers=None):
    """Waits for a zone to reach given status."""
    LOG.info('Waiting for zone %s to reach %s', zone_id, status)

    zone = client.show_zone(zone_id, headers=headers)[1]
    start = int(time.time())

    while zone['status'] != status:
        time.sleep(client.build_interval)
        zone = client.show_zone(zone_id, headers=headers)[1]
        status_curr = zone['status']
        if status_curr == status:
            LOG.info('Zone %s reached %s', zone_id, status)
            return

        if zone['status'] == const.ERROR:
            raise exceptions.InvalidStatusError('Zone', zone_id,
                                                zone['status'])

        if int(time.time()) - start >= client.build_timeout:
            message = ('Zone %(zone_id)s failed to reach status=%(status)s '
                       'within the required time (%(timeout)s s). Current '
                       'status: %(status_curr)s' %
                       {'zone_id': zone_id,
                        'status': status,
                        'status_curr': status_curr,
                        'timeout': client.build_timeout})

            caller = test_utils.find_test_caller()

            if caller:
                message = '(%s) %s' % (caller, message)

            raise lib_exc.TimeoutException(message)


def wait_for_zone_import_status(client, zone_import_id, status):
    """Waits for an imported zone to reach the given status."""
    LOG.info('Waiting for zone import %s to reach %s', zone_import_id, status)

    zone_import = client.show_zone_import(zone_import_id)[1]
    start = int(time.time())

    while zone_import['status'] != status:
        time.sleep(client.build_interval)
        zone_import = client.show_zone_import(zone_import_id)[1]
        status_curr = zone_import['status']
        if status_curr == status:
            LOG.info('Zone import %s reached %s', zone_import_id, status)
            return

        if zone_import['status'] == const.ERROR:
            raise exceptions.InvalidStatusError('Zone Import', zone_import_id,
                                                zone_import['status'])

        if int(time.time()) - start >= client.build_timeout:
            message = ('Zone import %(zone_import_id)s failed to reach '
                       'status=%(status)s within the required time '
                       '(%(timeout)s s). Current '
                       'status: %(status_curr)s' %
                       {'zone_import_id': zone_import_id,
                        'status': status,
                        'status_curr': status_curr,
                        'timeout': client.build_timeout})

            caller = test_utils.find_test_caller()

            if caller:
                message = '(%s) %s' % (caller, message)

            raise lib_exc.TimeoutException(message)


def wait_for_zone_export_status(client, zone_export_id, status, headers=None):
    """Waits for an exported zone to reach the given status."""
    LOG.info('Waiting for zone export %s to reach %s', zone_export_id, status)

    zone_export = client.show_zone_export(zone_export_id, headers=headers)[1]
    start = int(time.time())

    while zone_export['status'] != status:
        time.sleep(client.build_interval)
        zone_export = client.show_zone_export(
            zone_export_id, headers=headers)[1]
        status_curr = zone_export['status']
        if status_curr == status:
            LOG.info('Zone export %s reached %s', zone_export_id, status)
            return

        if zone_export['status'] == const.ERROR:
            raise exceptions.InvalidStatusError('Zone Export', zone_export_id,
                                                zone_export['status'])

        if int(time.time()) - start >= client.build_timeout:
            message = ('Zone export %(zone_export_id)s failed to reach '
                       'status=%(status)s within the required time '
                       '(%(timeout)s s). Current '
                       'status: %(status_curr)s' %
                       {'zone_export_id': zone_export_id,
                        'status': status,
                        'status_curr': status_curr,
                        'timeout': client.build_timeout})

            caller = test_utils.find_test_caller()

            if caller:
                message = '(%s) %s' % (caller, message)

            raise lib_exc.TimeoutException(message)


def wait_for_recordset_status(
        client, zone_id, recordset_id, status, headers=None):
    """Waits for a recordset to reach the given status."""
    LOG.info('Waiting for recordset %s to reach %s',
             recordset_id, status)

    recordset = client.show_recordset(
        zone_id, recordset_id, headers=headers)[1]
    start = int(time.time())

    while recordset['status'] != status:
        time.sleep(client.build_interval)
        recordset = client.show_recordset(
            zone_id, recordset_id, headers=headers)[1]
        status_curr = recordset['status']
        if status_curr == status:
            LOG.info('Recordset %s reached %s', recordset_id, status)
            return

        if recordset['status'] == const.ERROR:
            raise exceptions.InvalidStatusError('Recordset', recordset_id,
                                                recordset['status'])

        if int(time.time()) - start >= client.build_timeout:
            message = ('Recordset %(recordset_id)s failed to reach '
                       'status=%(status)s within the required time '
                       '(%(timeout)s s). Current '
                       'status: %(status_curr)s' %
                       {'recordset_id': recordset_id,
                        'status': status,
                        'status_curr': status_curr,
                        'timeout': client.build_timeout})

            caller = test_utils.find_test_caller()

            if caller:
                message = '(%s) %s' % (caller, message)

            raise lib_exc.TimeoutException(message)


def wait_for_query(client, name, rdatatype, found=True):
    """Query nameservers until the record of the given name and type is found.

    :param client: A QueryClient
    :param name: The record name for which to query
    :param rdatatype: The record type for which to query
    :param found: If True, wait until the record is found. Else, wait until the
        record disappears.
    """
    state = "found" if found else "removed"
    LOG.info("Waiting for record %s of type %s to be %s on nameservers %s",
             name, rdatatype, state, client.nameservers)
    start = int(time.time())

    while True:
        time.sleep(client.build_interval)

        responses = client.query(name, rdatatype)
        if found:
            all_answers_good = all(r.answer for r in responses)
        else:
            all_answers_good = all(not r.answer for r in responses)

        if all_answers_good:
            LOG.info("Record %s of type %s was successfully %s on nameservers "
                     "%s", name, rdatatype, state, client.nameservers)
            return

        if int(time.time()) - start >= client.build_timeout:
            message = ('Record %(name)s of type %(rdatatype)s not %(state)s '
                       'on nameservers %(nameservers)s within the required '
                       'time (%(timeout)s s)' %
                       {'name': name,
                        'rdatatype': rdatatype,
                        'state': state,
                        'nameservers': client.nameservers,
                        'timeout': client.build_timeout})

            caller = test_utils.find_test_caller()
            if caller:
                message = "(%s) %s" % (caller, message)

            raise lib_exc.TimeoutException(message)


def wait_for_ptr_status(client, fip_id, status):
    """Waits for a PTR associated with FIP to reach given status."""
    LOG.info('Waiting for PTR %s to reach %s', fip_id, status)

    ptr = client.show_ptr_record(fip_id)
    start = int(time.time())

    while ptr['status'] != status:
        time.sleep(client.build_interval)
        ptr = client.show_ptr_record(fip_id)
        status_curr = ptr['status']
        if status_curr == status:
            LOG.info('PTR %s reached %s', fip_id, status)
            return

        if ptr['status'] == const.ERROR:
            raise exceptions.InvalidStatusError('PTR', fip_id,
                                                ptr['status'])

        if int(time.time()) - start >= client.build_timeout:
            message = ('PTR for FIP: %(fip_id)s failed to reach '
                       'status=%(status)s within the required time '
                       '(%(timeout)s s). Current status: %(status_curr)s' %
                       {'fip_id': fip_id,
                        'status': status,
                        'status_curr': status_curr,
                        'timeout': client.build_timeout})

            caller = test_utils.find_test_caller()

            if caller:
                message = '(%s) %s' % (caller, message)

            raise lib_exc.TimeoutException(message)



# Copyright 2016 Rackspace
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_log import log as logging
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from tempest.lib.common.utils import data_utils

from designate_tempest_plugin.tests import base
from designate_tempest_plugin import data_utils as dns_data_utils

CONF = config.CONF
LOG = logging.getLogger(__name__)

quotas_types = ["api_export_size", "recordset_records",
                "zone_records", "zone_recordsets", "zones"]


class QuotasV2Test(base.BaseDnsV2Test):

    credentials = ["primary", "admin", "alt",
                   "project_member", "project_reader"]

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(QuotasV2Test, cls).setup_credentials()

    @classmethod
    def skip_checks(cls):
        super(QuotasV2Test, cls).skip_checks()

        if not CONF.dns_feature_enabled.api_v2_quotas:
            skip_msg = ("%s skipped as designate V2 Quotas API is not "
                        "available" % cls.__name__)
            raise cls.skipException(skip_msg)

    @classmethod
    def setup_clients(cls):
        super(QuotasV2Test, cls).setup_clients()

        cls.admin_client = cls.os_admin.dns_v2.QuotasClient()
        cls.quotas_client = cls.os_primary.dns_v2.QuotasClient()
        cls.alt_client = cls.os_alt.dns_v2.QuotasClient()

    def _store_quotas(self, project_id, cleanup=True):
        """Remember current quotas and reset them after the test"""
        params = {'project_id': project_id,
                  'headers': self.all_projects_header}

        _r, original_quotas = self.admin_client.show_quotas(**params)
        params.update(original_quotas)
        if cleanup:
            self.addCleanup(self.admin_client.update_quotas, **params)
        return original_quotas

    @decorators.idempotent_id('1dac991a-9e2e-452c-a47a-26ac37381ec5')
    def test_show_quotas(self):
        LOG.info("Show default quotas, validate all quota types exists and "
                 "their values are integers.")
        for user in ['primary', 'admin']:
            if user == 'primary':
                body = self.quotas_client.show_quotas()[1]
            if user == 'admin':
                body = self.admin_client.show_quotas(
                    project_id=self.quotas_client.project_id,
                    headers=self.all_projects_header)[1]
            for quota_type in quotas_types:
                self.assertIn(
                    quota_type, body.keys(),
                    'Failed, expected quota type:{} was not found '
                    'in received quota body'.format(quota_type))
            for quota_type, quota_value in body.items():
                self.assertTrue(
                    isinstance(quota_value, int),
                    'Failed, the value of:{} is:{}, expected integer'.format(
                        quota_type, quota_value))

        expected_allowed = ['os_admin', 'os_primary', 'os_alt']
        if CONF.dns_feature_enabled.enforce_new_defaults:
            expected_allowed.extend(['os_project_member', 'os_project_reader'])

        self.check_list_show_with_ID_RBAC_enforcement(
            'QuotasClient', 'show_quotas', expected_allowed, False)

    @decorators.idempotent_id('0448b089-5803-4ce3-8a6c-5c15ff75a2cc')
    def test_reset_quotas(self):
        self._store_quotas(project_id=self.quotas_client.project_id)

        LOG.info("Deleting (reset) quotas")

        expected_allowed = ['os_admin']

        self.check_CUD_RBAC_enforcement(
            'QuotasClient', 'delete_quotas', expected_allowed, False,
            project_id=self.quotas_client.project_id)

        body = self.admin_client.delete_quotas(
            project_id=self.quotas_client.project_id,
            headers=self.all_projects_header)[1]

        LOG.info("Ensuring an empty response body")
        self.assertEqual(body.strip(), b"")

    @decorators.idempotent_id('76d24c87-1b39-4e19-947c-c08e1380dc61')
    def test_update_quotas(self):
        self._store_quotas(project_id=self.quotas_client.project_id)
        LOG.info("Updating quotas")
        quotas = dns_data_utils.rand_quotas()
        body = self.admin_client.update_quotas(
            project_id=self.quotas_client.project_id,
            **quotas, headers=self.all_projects_header)[1]

        expected_allowed = ['os_admin']

        self.check_CUD_RBAC_enforcement(
            'QuotasClient', 'update_quotas', expected_allowed, False,
            project_id=self.quotas_client.project_id,
            **quotas, headers=self.all_projects_header)

        LOG.info("Ensuring the response has all quota types")
        self.assertExpected(quotas, body, [])

    @decorators.idempotent_id('9b09b3e2-7e88-4569-bce3-9be2f7ac70c3')
    def test_update_quotas_other_project(self):

        project_id = self.quotas_client.project_id
        self._store_quotas(project_id=project_id)

        LOG.info("Updating quotas for %s ", project_id)

        quotas = dns_data_utils.rand_quotas()
        request = quotas.copy()
        body = self.admin_client.update_quotas(
            project_id=project_id,
            headers=self.all_projects_header,
            **request)[1]

        LOG.info("Ensuring the response has all quota types")
        self.assertExpected(quotas, body, [])

        client_body = self.quotas_client.show_quotas(project_id=project_id)[1]

        self.assertExpected(quotas, client_body, [])

    @decorators.idempotent_id('21e45d30-dbc1-4173-9d6b-9b6813ef514b')
    def test_reset_quotas_other_project(self):

        LOG.info("Using 'alt' project id to set quotas on.")
        project_id = self.alt_client.tenant_id
        self._store_quotas(project_id=project_id)

        LOG.info("Resetting quotas to default for %s ", project_id)
        self.admin_client.delete_quotas(
            project_id=project_id,
            headers=self.all_projects_header)

        default_quotas = self.admin_client.show_quotas(
            project_id=project_id,
            headers=self.all_projects_header)[1]

        LOG.info("Updating quotas for %s ", project_id)

        quotas = dns_data_utils.rand_quotas()
        request = quotas.copy()
        self.admin_client.update_quotas(
            project_id=project_id,
            headers=self.all_projects_header,
            **request)

        self.admin_client.delete_quotas(
            project_id=project_id,
            headers=self.all_projects_header)

        final_quotas = self.admin_client.show_quotas(
            project_id=project_id,
            headers=self.all_projects_header)[1]

        self.assertExpected(default_quotas, final_quotas, [])

    @decorators.idempotent_id('9b09b3e2-7e88-4569-bce3-9be2f7ac70c4')
    def test_update_quotas_invalid_project(self):

        if not CONF.dns_feature_enabled.api_v2_quotas_verify_project:
            raise self.skipException("Project ID in quotas "
                                     "is not being verified.")
        original_quotas = self.quotas_client.show_quotas(
            project_id=self.quotas_client.project_id)[1]
        project_id = 'project-that-does-not-exist'

        LOG.info("Updating quotas for non-existing %s ", project_id)
        quotas = dns_data_utils.rand_quotas()
        request = quotas.copy()
        with self.assertRaisesDns(lib_exc.BadRequest, 'invalid_project', 400):
            self.admin_client.update_quotas(
                project_id=project_id,
                headers=self.all_projects_header,
                **request)

        LOG.info("Make sure that the quotas weren't changed")
        client_body = self.quotas_client.show_quotas(
            project_id=self.quotas_client.project_id)[1]
        self.assertExpected(original_quotas, client_body, [])


class QuotasV2TestNegative(base.BaseDnsV2Test):

    credentials = ["primary", "admin"]

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(QuotasV2TestNegative, cls).setup_credentials()

    @classmethod
    def skip_checks(cls):
        super(QuotasV2TestNegative, cls).skip_checks()

        if not CONF.dns_feature_enabled.api_v2_quotas:
            skip_msg = ("%s skipped as designate V2 Quotas API is not "
                        "available" % cls.__name__)
            raise cls.skipException(skip_msg)

    @classmethod
    def setup_clients(cls):
        super(QuotasV2TestNegative, cls).setup_clients()

        cls.admin_client = cls.os_admin.dns_v2.QuotasClient()
        cls.quotas_client = cls.os_primary.dns_v2.QuotasClient()

    @decorators.idempotent_id('ae82a0ba-da60-11eb-bf12-74e5f9e2a801')
    def test_admin_sets_quota_for_a_project(self):

        primary_project_id = self.quotas_client.project_id
        http_headers_to_use = [self.all_projects_header,
            {'x-auth-sudo-project-id': primary_project_id}]

        for http_header in http_headers_to_use:
            LOG.info('As Admin user set Zones quota for a Primary user and {} '
                     'HTTP header'.format(http_header))
            quotas = dns_data_utils.rand_quotas()
            self.admin_client.set_quotas(
                project_id=primary_project_id,
                quotas=quotas, headers=http_header)
            self.addCleanup(self.admin_client.delete_quotas,
                            project_id=primary_project_id)

            LOG.info("As Admin fetch the quotas for a Primary user")
            body = self.admin_client.show_quotas(
                project_id=primary_project_id, headers=http_header)[1]
            LOG.info('Ensuring that the "set" and "shown" quotas are same')
            self.assertExpected(quotas, body, [])

    @decorators.idempotent_id('40b9d7ac-da5f-11eb-bf12-74e5f9e2a801')
    def test_primary_fails_to_set_quota(self):

        primary_project_id = self.quotas_client.project_id
        LOG.info('Try to set quota as Primary user')
        self.assertRaises(
            lib_exc.Forbidden, self.quotas_client.set_quotas,
            project_id=primary_project_id,
            quotas=dns_data_utils.rand_quotas())

        LOG.info('Try to set quota as Primary user using '
                 '"x-auth-sudo-project-id" HTTP header')
        self.assertRaises(
            lib_exc.Forbidden, self.quotas_client.set_quotas,
            project_id=self.quotas_client.project_id,
            quotas=dns_data_utils.rand_quotas(),
            headers={'x-auth-sudo-project-id': primary_project_id})

        LOG.info('Try to set quota as Primary user using '
                 '"x-auth-all-projects" HTTP header')
        self.assertRaises(
            lib_exc.Forbidden, self.quotas_client.set_quotas,
            project_id=self.quotas_client.project_id,
            quotas=dns_data_utils.rand_quotas(),
            headers=self.all_projects_header)

    @decorators.idempotent_id('a6ce5b46-dcce-11eb-903e-74e5f9e2a801')
    def test_admin_sets_invalid_quota_values(self):

        primary_project_id = self.quotas_client.project_id

        for item in quotas_types:
            quota = dns_data_utils.rand_quotas()
            quota[item] = data_utils.rand_name()
            self.assertRaises(
                lib_exc.BadRequest, self.admin_client.set_quotas,
                project_id=primary_project_id,
                quotas=quota,
                headers=self.all_projects_header)

    @decorators.idempotent_id('ac212fd8-c602-11ec-b042-201e8823901f')
    def test_admin_sets_not_existing_quota_type(self):

        LOG.info('Try to set quota using not existing quota type in its body')
        primary_project_id = self.quotas_client.project_id
        quota = dns_data_utils.rand_quotas()
        quota[data_utils.rand_name()] = 777

        with self.assertRaisesDns(
                lib_exc.BadRequest, 'invalid_object', 400):
            self.admin_client.set_quotas(
                project_id=primary_project_id,
                quotas=quota, headers=self.all_projects_header)



# Copyright 2016 Rackspace
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_log import log as logging
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from designate_tempest_plugin.common import constants as const

from designate_tempest_plugin.tests import base
from designate_tempest_plugin import data_utils as dns_data_utils

CONF = config.CONF
LOG = logging.getLogger(__name__)


class ZonesTransferTest(base.BaseDnsV2Test):
    credentials = ['primary', 'alt', 'admin']

    @classmethod
    def setup_clients(cls):
        super(ZonesTransferTest, cls).setup_clients()
        cls.admin_zones_client = cls.os_admin.dns_v2.ZonesClient()
        cls.admin_accept_client = (
            cls.os_admin.dns_v2.TransferAcceptClient())
        cls.admin_tld_client = cls.os_admin.dns_v2.TldClient()
        cls.alt_zones_client = cls.os_alt.dns_v2.ZonesClient()
        cls.request_client = cls.os_primary.dns_v2.TransferRequestClient()
        cls.alt_request_client = cls.os_alt.dns_v2.TransferRequestClient()
        cls.accept_client = cls.os_primary.dns_v2.TransferAcceptClient()
        cls.alt_accept_client = cls.os_alt.dns_v2.TransferAcceptClient()

    @classmethod
    def resource_setup(cls):
        super(ZonesTransferTest, cls).resource_setup()

        # Make sure we have an allowed TLD available
        tld_name = dns_data_utils.rand_zone_name(name="ZonesTransferTest")
        cls.tld_name = f".{tld_name}"
        cls.class_tld = cls.admin_tld_client.create_tld(tld_name=tld_name[:-1])

    @classmethod
    def resource_cleanup(cls):
        cls.admin_tld_client.delete_tld(cls.class_tld[1]['id'])
        super(ZonesTransferTest, cls).resource_cleanup()

    @decorators.idempotent_id('60bd80ac-c979-4686-9a03-f2f775f272ab')
    def test_zone_transfer(self):
        LOG.info('Create a zone as primary tenant')
        zone_name = dns_data_utils.rand_zone_name(
            name="zone_transfer", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'],
                        ignore_errors=lib_exc.NotFound)

        LOG.info('Create a zone transfer_request for zone as primary tenant')
        transfer_request = (
            self.request_client.create_transfer_request_empty_body(
                zone['id'])[1])
        self.addCleanup(self.request_client.delete_transfer_request,
                        transfer_request['id'],
                        ignore_errors=lib_exc.NotFound)

        accept_data = {
                 "key": transfer_request['key'],
                 "zone_transfer_request_id": transfer_request['id']
        }

        LOG.info('Accept the request as alt tenant')
        transfer_accept = self.alt_accept_client.create_transfer_accept(
            accept_data)[1]

        LOG.info('Ensure we respond with COMPLETE status')
        show_request = self.request_client.show_transfer_request(
            transfer_request['id'])[1]
        self.assertEqual(const.COMPLETE, show_request['status'])

        LOG.info('Fetch the zone as alt tenant')
        alt_zone = self.alt_zones_client.show_zone(zone['id'])[1]
        self.addCleanup(self.wait_zone_delete,
                        self.alt_zones_client,
                        alt_zone['id'])

        LOG.info('Ensure 404 when fetching the zone as primary tenant')
        self.assertRaises(lib_exc.NotFound,
            lambda: self.zones_client.show_zone(zone['id']))

        LOG.info('Accept the request as admin tenant, should fail '
                 'with: "invalid_zone_transfer_request"')
        with self.assertRaisesDns(
                lib_exc.BadRequest, 'invalid_zone_transfer_request', 400):
            self.admin_accept_client.create_transfer_accept(accept_data)

        LOG.info('Delete the transfer_request')
        self.request_client.delete_transfer_request(transfer_request['id'])[1]

        LOG.info('Validation that transfer_accept deleted'
                 ' after the transfer_request delete')
        self.assertRaises(lib_exc.NotFound,
                          self.accept_client.show_transfer_accept,
                          transfer_accept['id'])

    @decorators.idempotent_id('5855b772-a036-11eb-9973-74e5f9e2a801')
    def test_zone_transfer_target_project(self):
        LOG.info('Create a zone as "primary" tenant')
        zone_name = dns_data_utils.rand_zone_name(
            name="zone_transfer_target_project", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]

        LOG.info('Create transfer_request with target project set to '
                 '"Admin" tenant')
        transfer_request_data = dns_data_utils.rand_transfer_request_data(
            target_project_id=self.os_admin.credentials.project_id)
        transfer_request = self.request_client.create_transfer_request(
            zone['id'], transfer_request_data)[1]
        self.addCleanup(self.request_client.delete_transfer_request,
                        transfer_request['id'])
        LOG.info('Ensure we respond with ACTIVE status')
        self.assertEqual(const.ACTIVE, transfer_request['status'])

        LOG.info('Accept the request as "alt" tenant, Expected: should fail '
                 'as "admin" was set as a target project.')
        accept_data = {
                 "key": transfer_request['key'],
                 "zone_transfer_request_id": transfer_request['id']
        }
        self.assertRaises(
            lib_exc.Forbidden, self.alt_accept_client.create_transfer_accept,
            transfer_accept_data=accept_data)

        LOG.info('Accept the request as "Admin" tenant, Expected: should work')
        self.admin_accept_client.create_transfer_accept(
            accept_data, headers={'x-auth-sudo-project-id':
                                  self.os_admin.credentials.project_id},
            extra_headers=True)
        LOG.info('Fetch the zone as "Admin" tenant')
        admin_zone = self.admin_zones_client.show_zone(
            zone['id'], headers={'x-auth-sudo-project-id':
                                 self.os_admin.credentials.project_id})[1]
        self.addCleanup(self.wait_zone_delete,
                        self.admin_zones_client,
                        admin_zone['id'])



# Copyright 2016 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import testtools
from oslo_log import log as logging
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from designate_tempest_plugin.tests import base
from designate_tempest_plugin import data_utils as dns_data_utils

CONF = config.CONF
LOG = logging.getLogger(__name__)


class BaseTsigkeyTest(base.BaseDnsV2Test):
    excluded_keys = ['created_at', 'updated_at', 'links']

    @classmethod
    def setup_clients(cls):
        super(BaseTsigkeyTest, cls).setup_clients()

        cls.admin_tld_client = cls.os_admin.dns_v2.TldClient()

    @classmethod
    def resource_setup(cls):
        super(BaseTsigkeyTest, cls).resource_setup()

        # Make sure we have an allowed TLD available
        tld_name = dns_data_utils.rand_zone_name(name="BaseTsigkeyTest")
        cls.tld_name = f".{tld_name}"
        cls.class_tld = cls.admin_tld_client.create_tld(tld_name=tld_name[:-1])

    @classmethod
    def resource_cleanup(cls):
        cls.admin_tld_client.delete_tld(cls.class_tld[1]['id'])
        super(BaseTsigkeyTest, cls).resource_cleanup()


class TsigkeyAdminTest(BaseTsigkeyTest):
    credentials = ["primary", "admin", "project_member",
                   "project_reader", "alt"]

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(TsigkeyAdminTest, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(TsigkeyAdminTest, cls).setup_clients()
        cls.admin_client = cls.os_admin.dns_v2.TsigkeyClient()
        cls.pool_admin_client = cls.os_admin.dns_v2.PoolClient()

        cls.primary_client = cls.os_primary.dns_v2.TsigkeyClient()

    @decorators.idempotent_id('e7b484e3-7ed5-4840-89d7-1e696986f8e4')
    def test_create_tsigkey_for_zone(self):
        LOG.info('Create a resource')
        zone_name = dns_data_utils.rand_zone_name(
            name="create_tsigkey_for_zone", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])

        tsigkey_data = {
                        "name": dns_data_utils.rand_zone_name(
                            'test_create_tsigkey_for_zone'),
                        "algorithm": "hmac-sha256",
                        "secret": "SomeSecretKey",
                        "scope": "ZONE",
                        "resource_id": zone['id']}

        LOG.info('Create a tsigkey')
        tsigkey = self.admin_client.create_tsigkey(
                         tsigkey_data['resource_id'],
                         tsigkey_data['name'], tsigkey_data['algorithm'],
                         tsigkey_data['secret'], tsigkey_data['scope'])[1]
        self.addCleanup(self.admin_client.delete_tsigkey, tsigkey['id'])

        self.assertEqual(tsigkey_data["name"], tsigkey['name'])
        self.assertEqual(tsigkey_data["scope"], 'ZONE')

    @decorators.idempotent_id('45975fa6-d726-11eb-beba-74e5f9e2a801')
    def test_create_tsigkey_for_pool(self):
        LOG.info('Get the valid pool ID from list of pools')
        pool = self.pool_admin_client.list_pools()[1]['pools'][0]

        LOG.info('Create a tsigkey')
        tsigkey_data = {
                        "name": dns_data_utils.rand_zone_name('Example_Key'),
                        "algorithm": "hmac-sha256",
                        "secret": "SomeSecretKey",
                        "scope": "POOL",
                        "resource_id": pool['id']}
        tsigkey = self.admin_client.create_tsigkey(
                         tsigkey_data['resource_id'],
                         tsigkey_data['name'], tsigkey_data['algorithm'],
                         tsigkey_data['secret'], tsigkey_data['scope'])[1]
        self.addCleanup(self.admin_client.delete_tsigkey, tsigkey['id'])
        self.assertEqual(tsigkey_data["name"], tsigkey['name'])
        self.assertEqual(tsigkey_data["scope"], 'POOL')

        # Test RBAC
        expected_allowed = ['os_admin']

        self.check_CUD_RBAC_enforcement(
            'TsigkeyClient', 'create_tsigkey', expected_allowed, False,
            tsigkey_data['resource_id'],
            tsigkey_data['name'], tsigkey_data['algorithm'],
            tsigkey_data['secret'], tsigkey_data['scope'])

    @decorators.idempotent_id('d46e5e86-a18c-4315-aa0c-95a00e816fbf')
    def test_list_tsigkey(self):
        LOG.info('Create a resource')
        zone_name = dns_data_utils.rand_zone_name(
            name="list_tsigkey", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])
        LOG.info('Create a tsigkey')
        tsigkey = self.admin_client.create_tsigkey(resource_id=zone['id'])[1]
        self.addCleanup(self.admin_client.delete_tsigkey, tsigkey['id'])
        body = self.admin_client.list_tsigkeys()[1]
        self.assertGreater(len(body['tsigkeys']), 0)

        # Test RBAC
        expected_allowed = ['os_admin']

        self.check_list_IDs_RBAC_enforcement(
            'TsigkeyClient', 'list_tsigkeys', expected_allowed,
            [tsigkey['id']])

    @decorators.idempotent_id('d46e5e86-a18c-4315-aa0c-95a00e816fbf')
    def test_list_tsigkeys_limit_results(self):
        for i in range(3):
            LOG.info('As Primary user create a zone: {} '.format(i))
            zone_name = dns_data_utils.rand_zone_name(
                name="list_tsigkey_limit", suffix=self.tld_name)
            zone = self.zones_client.create_zone(name=zone_name)[1]
            self.addCleanup(
                self.wait_zone_delete, self.zones_client, zone['id'])
            LOG.info('As Admin user create a tsigkey: {} '.format(i))
            tsigkey = self.admin_client.create_tsigkey(
                resource_id=zone['id'])[1]
            self.addCleanup(self.admin_client.delete_tsigkey, tsigkey['id'])
        LOG.info('As Admin client, list all tsigkey using '
                 'URL query: "limit=2"')
        body = self.admin_client.list_tsigkeys(params={'limit': 2})[1]
        self.assertEqual(len(body['tsigkeys']), 2)

    @decorators.idempotent_id('f31447b0-d817-11eb-b95a-74e5f9e2a801')
    def test_list_tsigkeys_using_marker(self):
        test_tsigkeys_name = 'marker_tsigkey_'
        test_tsigkeys_names = [test_tsigkeys_name + str(i) for i in range(4)]

        LOG.info('Create tsigkeys named: {}'.format(test_tsigkeys_names))
        created_tsigkeys = []
        for name in test_tsigkeys_names:
            LOG.info('As Primary user create a zone to be used '
                     'for {}'.format(name))
            zone_name = dns_data_utils.rand_zone_name(
                name="list_tsigkey_marker", suffix=self.tld_name)
            zone = self.zones_client.create_zone(name=zone_name)[1]
            self.addCleanup(
                self.wait_zone_delete, self.zones_client, zone['id'])
            LOG.info('As Admin user create "{}" tsigkey'.format(name))
            tsigkey = self.admin_client.create_tsigkey(
                resource_id=zone['id'], name=name)[1]
            self.addCleanup(self.admin_client.delete_tsigkey, tsigkey['id'])
            created_tsigkeys.append(tsigkey['id'])

        LOG.info('As Admin, list all tsigkeys using url queries:"limit=2" '
                 'and "name={}*"'.format(test_tsigkeys_name))
        body = self.admin_client.list_tsigkeys(
            params={
                'limit': 2, 'name': test_tsigkeys_name + '*'})[1]
        tsigkeys = body['tsigkeys']
        self.assertEqual(2, len(tsigkeys),
                         'Failed, response is not limited as expected')

        LOG.info('Get the marker to be used for subsequent request')
        first_set_of_ids = [item['id'] for item in tsigkeys]
        links = body['links']
        marker = links['next'].split('marker=')[-1]

        LOG.info('Use marker for subsequent request to get the rest of '
                 'tsigkeys that contains "{}*" in their '
                 'names'.format(test_tsigkeys_name))
        tsigkeys = self.admin_client.list_tsigkeys(
            params={'marker': marker, 'limit': 2,
                    'name': test_tsigkeys_name + '*'})[1]['tsigkeys']
        self.assertEqual(2, len(tsigkeys),
                         'Failed, response is not limited as expected')
        second_set_of_ids = [item['id'] for item in tsigkeys]

        LOG.info('Make sure that the merge of tsigkeys IDs received in two '
                 'phases using "marker" url query, contains all the IDs '
                 'created within the test')
        self.assertEqual(
            sorted(first_set_of_ids + second_set_of_ids),
            sorted(created_tsigkeys),
            'Failed, tsigkeys IDs received in two phases are not as expected')

    @decorators.idempotent_id('d5c6dfcc-d8af-11eb-b95a-74e5f9e2a801')
    def test_list_tsigkey_sort_key_with_sort_direction(self):
        names_to_create = [data_utils.rand_name(name) for name in
                           ['bbb_tsgikey', 'aaa_tsgikey', 'ccc_tsgikey']]
        created_tsigkey_ids = []
        for name in names_to_create:
            LOG.info('As Primary user create a zone for: {} '.format(name))
            zone_name = dns_data_utils.rand_zone_name(
                name="list_tsigkey_sort", suffix=self.tld_name)
            zone = self.zones_client.create_zone(name=zone_name)[1]
            self.addCleanup(
                self.wait_zone_delete, self.zones_client, zone['id'])
            LOG.info('As Admin user create a tsigkey: {} '.format(name))
            tsigkey = self.admin_client.create_tsigkey(
                resource_id=zone['id'], name=name)[1]
            self.addCleanup(self.admin_client.delete_tsigkey, tsigkey['id'])
            created_tsigkey_ids.append(tsigkey['id'])

        LOG.info('As Admin, list all tsigkeys using "asc" to sort by names')
        sorted_tsigkeys = self.admin_client.list_tsigkeys(
            params={'sort_dir': 'asc', 'sort_key': 'name'})[1]['tsigkeys']
        sorted_by_names = [item['name'] for item in sorted_tsigkeys]
        self.assertEqual(
            sorted(sorted_by_names),
            sorted_by_names,
            'Failed, tsgikeys names are not sorted in "asc" as expected')

        LOG.info('As Admin, list all tsigkey using "desc" to sort by names')
        sorted_tsigkeys = self.admin_client.list_tsigkeys(
            params={'sort_dir': 'desc', 'sort_key': 'name'})[1]['tsigkeys']
        sorted_by_names = [item['name'] for item in sorted_tsigkeys]
        self.assertEqual(
            sorted(sorted_by_names, reverse=True),
            sorted_by_names,
            'Failed, tsgikeys names are not sorted in "desc" as expected')

        LOG.info('As Admin, list all tsigkeys using "asc" to sort by ID')
        sorted_tsigkeys = self.admin_client.list_tsigkeys(
            params={'sort_dir': 'asc', 'sort_key': 'id'})[1]['tsigkeys']
        sorted_by_ids = [item['id'] for item in sorted_tsigkeys]
        self.assertEqual(
            sorted(sorted_by_ids),
            sorted_by_ids,
            'Failed, tsgikeys IDs are not sorted in "asc" as expected')

        LOG.info('As Admin, list all tsigkeys using "zababun" direction '
                 'to sort by names, expected: "invalid_sort_dir"')
        self.assertRaisesDns(
            lib_exc.BadRequest, 'invalid_sort_dir', 400,
            self.admin_client.list_tsigkeys,
            params={'sort_dir': 'zababun', 'sort_key': 'name'})

        LOG.info('As Admin, list all tsigkeys using "zababun" as a key value,'
                 'expected: "invalid_sort_key"')
        self.assertRaisesDns(
            lib_exc.BadRequest, 'invalid_sort_key', 400,
            self.admin_client.list_tsigkeys,
            params={'sort_dir': 'asc', 'sort_key': 'zababun'})

    @decorators.idempotent_id('4162a840-d8b2-11eb-b95a-74e5f9e2a801')
    def test_list_tsigkey_filter_by_name(self):
        tsigkey_name = data_utils.rand_name('ddd_tsgikey')
        LOG.info('As Primary user create a zone for: {} '.format(tsigkey_name))
        zone_name = dns_data_utils.rand_zone_name(
            name="list_tsigkey_filter_name", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])
        LOG.info('As Admin user create a tsigkey: {} '.format(tsigkey_name))
        tsigkey = self.admin_client.create_tsigkey(
            resource_id=zone['id'], name=tsigkey_name)[1]
        self.addCleanup(self.admin_client.delete_tsigkey, tsigkey['id'])

        LOG.info('As Admin, list all tsigkeys named:{}'.format(tsigkey_name))
        listed_tsigkeys = self.admin_client.list_tsigkeys(
            params={'name': tsigkey_name})[1]['tsigkeys']
        self.assertEqual(
            1, len(listed_tsigkeys),
            'Failed, only a single tsigkey, named: {} should be '
            'listed.'.format(tsigkey_name))

        LOG.info('As Admin, list all tsigkeys named:"zababun"')
        listed_tsigkeys = self.admin_client.list_tsigkeys(
            params={'name': 'zababun'})[1]['tsigkeys']
        self.assertEqual(
            0, len(listed_tsigkeys), 'Failed, no tsigkey should be listed')

    @decorators.idempotent_id('e8bcf80a-d8b4-11eb-b95a-74e5f9e2a801')
    @testtools.skipIf(CONF.dns_feature_enabled.test_multipool_with_delete_opt,
                      'Multipools feature is being tested with --delete '
                      'option. It might delete pools that were created in '
                      'other tests.')
    def test_list_tsigkey_filter_by_scope(self):

        LOG.info('Create tsigkey for a pool')
        pool = self.pool_admin_client.create_pool(
            project_id=self.os_admin.credentials.project_id)[1]
        self.addCleanup(
            self.pool_admin_client.delete_pool, pool['id'],
            headers={
                'x-auth-sudo-project-id':
                    self.os_admin.credentials.project_id})
        pool_tsigkey = self.admin_client.create_tsigkey(
            resource_id=pool['id'], scope='POOL')[1]
        self.addCleanup(self.admin_client.delete_tsigkey, pool_tsigkey['id'])

        LOG.info('Create tsigkey for a zone')
        zone_name = dns_data_utils.rand_zone_name(
            name="list_tsigkey_filter_scope", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])
        zone_tsigkey = self.admin_client.create_tsigkey(
            resource_id=zone['id'], scope='ZONE')[1]
        self.addCleanup(self.admin_client.delete_tsigkey, zone_tsigkey['id'])

        LOG.info('List all "scope=POOL" tsigkeys')
        listed_pool_scopes = [
            item['scope'] for item in self.admin_client.list_tsigkeys(
                params={'scope': 'POOL'})[1]['tsigkeys']]
        self.assertEqual(
            {'POOL'}, set(listed_pool_scopes),
            'Failed, the only scopes expected to be listed are: "POOL"')

        LOG.info('List all "scope=ZONE" tsigkeys')
        listed_zone_scopes = [
            item['scope'] for item in self.admin_client.list_tsigkeys(
                params={'scope': 'ZONE'})[1]['tsigkeys']]
        self.assertEqual(
            {'ZONE'}, set(listed_zone_scopes),
            'Failed, the only scopes expected to be listed are: "ZONE"')

        LOG.info('List all "scope=zababun" tsigkeys')
        listed_zone_scopes = [
            item['scope'] for item in self.admin_client.list_tsigkeys(
                params={'scope': 'zababun'})[1]['tsigkeys']]
        self.assertEqual(
            0, len(listed_zone_scopes),
            'Failed, no tsigkey is expected to be listed')

    @decorators.idempotent_id('794554f0-d8b8-11eb-b95a-74e5f9e2a801')
    @testtools.skipIf(CONF.dns_feature_enabled.test_multipool_with_delete_opt,
                      'Multipools feature is being tested with --delete '
                      'option. It might delete pools that were created in '
                      'other tests.')
    def test_list_tsigkey_filter_by_algorithm(self):

        LOG.info('Create tsigkey for a pool')
        algorithm = 'hmac-sha256'
        pool = self.pool_admin_client.create_pool(
            project_id=self.os_admin.credentials.project_id)[1]
        self.addCleanup(
            self.pool_admin_client.delete_pool, pool['id'],
            headers={
                'x-auth-sudo-project-id':
                    self.os_admin.credentials.project_id})
        pool_tsigkey = self.admin_client.create_tsigkey(
            resource_id=pool['id'], algorithm=algorithm)[1]
        self.addCleanup(self.admin_client.delete_tsigkey, pool_tsigkey['id'])

        LOG.info('List all "algorithm={}" tsigkeys '.format(algorithm))
        listed_tsigkeys = [
            item['algorithm'] for item in self.admin_client.list_tsigkeys(
                params={'algorithm': algorithm})[1]['tsigkeys']]
        self.assertEqual(
            {algorithm}, set(listed_tsigkeys),
            'Failed, the only tsigkeys expected to be listed must '
            'have algorithm:{} '.format(algorithm))

        LOG.info('List all "algorithm=zababun" tsigkeys')
        listed_tsigkeys = [
            item['algorithm'] for item in self.admin_client.list_tsigkeys(
                params={'algorithm': 'zababun'})[1]['tsigkeys']]
        self.assertEqual(
            0, len(listed_tsigkeys),
            "Failed, no tsigkey is expectedto be listed")

    @decorators.idempotent_id('c5d7facf-0f05-47a2-a4fb-87f203860880')
    def test_show_tsigkey(self):
        LOG.info('Create a resource')
        zone_name = dns_data_utils.rand_zone_name(
            name="show_tsigkey", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])

        LOG.info('Create a tsigkey')
        tsigkey = self.admin_client.create_tsigkey(resource_id=zone['id'])[1]
        self.addCleanup(self.admin_client.delete_tsigkey, tsigkey['id'])

        LOG.info('Fetch the tsigkey')
        body = self.admin_client.show_tsigkey(tsigkey['id'])[1]

        LOG.info('Ensure the fetched response matches the created tsigkey')
        self.assertExpected(tsigkey, body, self.excluded_keys)

        # Test RBAC
        expected_allowed = ['os_admin']

        self.check_list_show_RBAC_enforcement(
            'TsigkeyClient', 'show_tsigkey', expected_allowed, True,
            tsigkey['id'])

    @decorators.idempotent_id('d09dc0dd-dd72-41ee-9085-2afb2bf35459')
    def test_update_tsigkey(self):
        LOG.info('Create a resource')
        zone_name = dns_data_utils.rand_zone_name(
            name="update_tsigkey", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])

        LOG.info('Create a tsigkey')
        tsigkey = self.admin_client.create_tsigkey(resource_id=zone['id'])[1]
        self.addCleanup(self.admin_client.delete_tsigkey, tsigkey['id'])

        tsigkey_data = {
                        "name": "Patch tsigkey",
                        "secret": "NewSecretKey"}

        LOG.info('Update the tsigkey')
        patch_tsigkey = self.admin_client.update_tsigkey(tsigkey['id'],
                               name=tsigkey_data['name'],
                               secret=tsigkey_data['secret'])[1]

        self.assertEqual(tsigkey_data['name'], patch_tsigkey['name'])
        self.assertEqual(tsigkey_data['secret'], patch_tsigkey['secret'])

        # Test RBAC
        expected_allowed = ['os_admin']

        self.check_CUD_RBAC_enforcement(
            'TsigkeyClient', 'update_tsigkey', expected_allowed, False,
            tsigkey['id'], name=tsigkey_data['name'],
            secret=tsigkey_data['secret'])

    @decorators.idempotent_id('9cdffbd2-bc67-4a25-8eb7-4be8635c88a3')
    def test_delete_tsigkey(self):
        LOG.info('Create a resource')
        zone_name = dns_data_utils.rand_zone_name(
            name="delete_tsigkey", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])

        LOG.info('Create a tsigkey')
        tsigkey = self.admin_client.create_tsigkey(resource_id=zone['id'])[1]

        # Test RBAC
        expected_allowed = ['os_admin']

        self.check_CUD_RBAC_enforcement(
            'TsigkeyClient', 'delete_tsigkey', expected_allowed, False,
            tsigkey['id'])

        LOG.info('Delete the tsigkey')
        self.admin_client.delete_tsigkey(tsigkey['id'])

        self.assertRaises(lib_exc.NotFound,
           lambda: self.admin_client.show_tsigkey(tsigkey['id']))

    @decorators.idempotent_id('4bdc20ef-96f9-47f6-a1aa-275159af326b')
    def test_list_tsigkeys_dot_json_fails(self):
        uri = self.admin_client.get_uri('tsigkeys.json')

        self.assertRaises(lib_exc.NotFound,
            lambda: self.admin_client.get(uri))


class TestTsigkeyNotFoundAdmin(BaseTsigkeyTest):

    credentials = ["admin", "primary"]

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(TestTsigkeyNotFoundAdmin, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(TestTsigkeyNotFoundAdmin, cls).setup_clients()
        cls.admin_client = cls.os_admin.dns_v2.TsigkeyClient()

    @decorators.idempotent_id('824c9b49-edc5-4282-929e-467a158d23e4')
    def test_show_tsigkey_404(self):
        e = self.assertRaises(lib_exc.NotFound,
                              self.admin_client.show_tsigkey,
                              data_utils.rand_uuid())
        self.assertTsigkey404(e.resp, e.resp_body)

    @decorators.idempotent_id('4ef3493a-ee66-4c62-b070-c57fa9568b69')
    def test_update_tsigkey_404(self):
        e = self.assertRaises(lib_exc.NotFound,
                              self.admin_client.update_tsigkey,
                              data_utils.rand_uuid())
        self.assertTsigkey404(e.resp, e.resp_body)

    @decorators.idempotent_id('ba438ede-4823-4922-8f4c-8de278f3d454')
    def test_delete_tsigkey_404(self):
        e = self.assertRaises(lib_exc.NotFound,
                              self.admin_client.delete_tsigkey,
                              data_utils.rand_uuid())
        self.assertTsigkey404(e.resp, e.resp_body)

    def assertTsigkey404(self, resp, resp_body):
        self.assertEqual(404, resp.status)
        self.assertEqual(404, resp_body['code'])
        self.assertEqual("tsigkey_not_found", resp_body['type'])
        self.assertEqual("Could not find TsigKey", resp_body['message'])


class TestTsigkeyInvalidIdAdmin(BaseTsigkeyTest):

    credentials = ["admin", "primary"]

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(TestTsigkeyInvalidIdAdmin, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(TestTsigkeyInvalidIdAdmin, cls).setup_clients()
        cls.admin_client = cls.os_admin.dns_v2.TsigkeyClient()
        cls.pool_admin_client = cls.os_admin.dns_v2.PoolClient()

    @decorators.idempotent_id('2a8dfc75-9884-4b1c-8f1f-ed835d96f2fe')
    def test_show_tsigkey_invalid_uuid(self):
        e = self.assertRaises(lib_exc.BadRequest,
                              self.admin_client.show_tsigkey,
                              'foo')
        self.assertTsigkeyInvalidId(e.resp, e.resp_body)

    @decorators.idempotent_id('2befa10f-fc42-4ae9-9276-672e23f045f2')
    def test_update_tsigkey_invalid_uuid(self):
        e = self.assertRaises(lib_exc.BadRequest,
                              self.admin_client.update_tsigkey,
                              'foo')
        self.assertTsigkeyInvalidId(e.resp, e.resp_body)

    @decorators.idempotent_id('55c2fea0-ead6-44c7-8bb1-05111412afdd')
    def test_delete_tsigkey_invalid_uuid(self):
        e = self.assertRaises(lib_exc.BadRequest,
                              self.admin_client.delete_tsigkey,
                              'foo')
        self.assertTsigkeyInvalidId(e.resp, e.resp_body)

    def assertTsigkeyInvalidId(self, resp, resp_body):
        self.assertEqual(400, resp.status)
        self.assertEqual(400, resp_body['code'])
        self.assertEqual("invalid_uuid", resp_body['type'])
        self.assertEqual("Invalid UUID tsigkey_id: foo",
                         resp_body['message'])

    @decorators.idempotent_id('f94af13a-d743-11eb-beba-74e5f9e2a801')
    def test_create_tsigkey_for_zone_invalid_algorithm(self):
        zone_name = dns_data_utils.rand_zone_name(
            name="create_tsigkey_invalid_algo", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])
        tsigkey_data = {
                        "name": dns_data_utils.rand_zone_name('Example_Key'),
                        "algorithm": "zababun",
                        "secret": "SomeSecretKey",
                        "scope": "ZONE",
                        "resource_id": zone['id']}
        self.assertRaisesDns(
            lib_exc.BadRequest, 'invalid_object', 400,
            self.admin_client.create_tsigkey,
            tsigkey_data['resource_id'],
            tsigkey_data['name'], tsigkey_data['algorithm'],
            tsigkey_data['secret'], tsigkey_data['scope'])

    @decorators.idempotent_id('4df903d8-d745-11eb-beba-74e5f9e2a801')
    def test_create_tsigkey_for_zone_invalid_name(self):
        LOG.info('Create a zone resource')
        zone_name = dns_data_utils.rand_zone_name(
            name="create_tsigkey_invalid_name", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])
        tsigkey_data = {
                        "name": dns_data_utils.rand_zone_name(
                            'Example_Key') * 1000,
                        "algorithm": "hmac-sha256",
                        "secret": "SomeSecretKey",
                        "scope": "ZONE",
                        "resource_id": zone['id']}
        self.assertRaisesDns(
            lib_exc.BadRequest, 'invalid_object', 400,
            self.admin_client.create_tsigkey,
            tsigkey_data['resource_id'],
            tsigkey_data['name'], tsigkey_data['algorithm'],
            tsigkey_data['secret'], tsigkey_data['scope'])

    @decorators.idempotent_id('5d6b8a84-d745-11eb-beba-74e5f9e2a801')
    @decorators.skip_because(bug="1933760")
    def test_create_tsigkey_for_zone_empty_secret(self):
        LOG.info('Create a zone resource')
        zone_name = dns_data_utils.rand_zone_name(
            name="create_tsigkey_empty_secret", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])
        tsigkey_data = {
                        "name": dns_data_utils.rand_zone_name('Example_Key'),
                        "algorithm": "hmac-sha256",
                        "secret": '',
                        "scope": "ZONE",
                        "resource_id": zone['id']}
        self.assertRaisesDns(
            lib_exc.BadRequest, 'invalid_object', 400,
            self.admin_client.create_tsigkey,
            tsigkey_data['resource_id'],
            tsigkey_data['name'], tsigkey_data['algorithm'],
            tsigkey_data['secret'], tsigkey_data['scope'])

    @decorators.idempotent_id('dfca9268-d745-11eb-beba-74e5f9e2a801')
    def test_create_tsigkey_for_zone_invalid_scope(self):
        LOG.info('Create a zone resource')
        zone_name = dns_data_utils.rand_zone_name(
            name="create_tsigkey_invalid_scope", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])
        tsigkey_data = {
                        "name": dns_data_utils.rand_zone_name('Example_Key'),
                        "algorithm": "hmac-sha256",
                        "secret": "SomeSecretKey",
                        "scope": "zababun",
                        "resource_id": zone['id']}
        self.assertRaisesDns(
            lib_exc.BadRequest, 'invalid_object', 400,
            self.admin_client.create_tsigkey,
            tsigkey_data['resource_id'],
            tsigkey_data['name'], tsigkey_data['algorithm'],
            tsigkey_data['secret'], tsigkey_data['scope'])

    @decorators.idempotent_id('57255858-d74a-11eb-beba-74e5f9e2a801')
    def test_create_tsigkey_for_zone_invalid_zone_id(self):
        LOG.info('Create a resource')
        zone_name = dns_data_utils.rand_zone_name(
            name="create_tsigkey_invalide_zone_id", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])
        tsigkey_data = {
                        "name": dns_data_utils.rand_zone_name('Example_Key'),
                        "algorithm": "hmac-sha256",
                        "secret": "SomeSecretKey",
                        "scope": "ZONE",
                        "resource_id": data_utils.rand_uuid}
        self.assertRaisesDns(
            lib_exc.BadRequest, 'invalid_object', 400,
            self.admin_client.create_tsigkey,
            tsigkey_data['resource_id'],
            tsigkey_data['name'], tsigkey_data['algorithm'],
            tsigkey_data['secret'], tsigkey_data['scope'])

    @decorators.idempotent_id('0dfbc2f8-d8bb-11eb-b95a-74e5f9e2a801')
    @testtools.skipIf(CONF.dns_feature_enabled.test_multipool_with_delete_opt,
                      'Multipools feature is being tested with --delete '
                      'option. It might delete pools that were created in '
                      'other tests.')
    @decorators.skip_because(bug="1934120")
    def test_create_tsigkey_for_pool_with_scope_zone(self):
        pool = self.pool_admin_client.create_pool()[1]
        self.addCleanup(self.pool_admin_client.delete_pool, pool['id'])

        LOG.info('Try to create a tsigkey using pool ID and "scope:ZONE", '
                 'should fail because ID is for pool, but scope is ZONE')
        self.assertRaisesDns(
            lib_exc.BadRequest, 'invalid_object', 400,
            self.admin_client.create_tsigkey,
            resource_id=pool['id'], scope='ZONE')



"""
Copyright 2016 Rackspace

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from oslo_log import log as logging
from oslo_utils import versionutils

from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions

from designate_tempest_plugin.tests import base
from designate_tempest_plugin.common import waiters
from designate_tempest_plugin import data_utils as dns_data_utils


CONF = config.CONF
LOG = logging.getLogger(__name__)
RECORDSETS_DATASET = [
    'A',
    'AAAA',
    'CNAME',
    'MX',
    'SPF',
    'SRV',
    'SSHFP',
    'TXT',
]
INVALID_TXT_DATASET = {
    "trailing_slash": {"data": "\\"},
    "trailing_double_slash": {"data": "\\\\"},
    "trailing_slash_after_text": {"data": "testtext\\"}}
VALID_TXT_DATASET = {
    "slash_with_one_trailing_space": {"data": "\"\\ \""},
    "slash_with_many_trailing_space": {"data": "\"\\    \""},
    "text_with_slash_and_trailing_space": {"data": "\"the txts    \""}}
INVALID_MX_DATASET = {
    "empty_preference": {"pref": ""},
    "minus_zero_preference": {"pref": "-0"},
    "minus_one_preference": {"pref": "-1"}}
INVALID_SSHFP_DATASET = {
    "minus_zero_algorithm": {"algo": "-0", "finger": None},
    "minus_zero_fingerprint": {"algo": None, "finger": "-0"},
    "minus_one_algorithm": {"algo": "-1", "finger": None},
    "minus_one_fingerprint": {"algo": None, "finger": "-1"}}


class RecordsetValidationTest(base.BaseDnsV2Test):

    credentials = ["admin", "primary"]

    def setUp(self):
        super(RecordsetValidationTest, self).setUp()
        self._zone = None

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(RecordsetValidationTest, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(RecordsetValidationTest, cls).setup_clients()

        cls.admin_tld_client = cls.os_admin.dns_v2.TldClient()
        cls.recordset_client = cls.os_primary.dns_v2.RecordsetClient()

    @property
    def zone(self):
        if self._zone is None:
            tld_name = dns_data_utils.rand_zone_name(
                name="recordsetvalidation")
            self.class_tld = self.admin_tld_client.create_tld(
                tld_name=tld_name[:-1])
            self.addCleanup(
                self.admin_tld_client.delete_tld, self.class_tld[1]['id'])
            zone_name = dns_data_utils.rand_zone_name(name="TestZone",
                                                  suffix=f'.{tld_name}')
            zone_data = dns_data_utils.rand_zone_data(name=zone_name)
            resp, body = self.zones_client.create_zone(**zone_data)
            self._zone = body
            self.addCleanup(self.wait_zone_delete,
                            self.zones_client,
                            body['id'])
        return self._zone

    def create_recordset(self, data):
        resp, body = self.recordset_client.create_recordset(
            self.zone['id'], data)

        return body

    @decorators.idempotent_id('c5ef87e2-cb79-4758-b968-18eef2c251df')
    def test_create_invalid(self):
        for rtype in RECORDSETS_DATASET:
            data = ["b0rk"]
            for i in data:
                model = dns_data_utils.make_rand_recordset(
                    self.zone['name'], rtype)
                model['data'] = i
                self.assertRaisesDns(
                    exceptions.BadRequest, 'invalid_object', 400,
                    self.recordset_client.create_recordset,
                    self.zone['id'], model
                )

    @decorators.idempotent_id('1164c826-dceb-4557-9a22-7d65c4a4f5f4')
    def test_update_invalid(self):
        for rtype in RECORDSETS_DATASET:
            data = ["b0rk"]
            post_model = dns_data_utils.make_rand_recordset(
                self.zone['name'], rtype)
            recordset = self.create_recordset(post_model)
            for i in data:
                model = dns_data_utils.make_rand_recordset(
                    self.zone['name'], rtype)
                model['data'] = i
                self.assertRaisesDns(
                    exceptions.BadRequest, 'invalid_object', 400,
                    self.recordset_client.update_recordset,
                    self.zone['id'], recordset['id'], model
                )

    @decorators.idempotent_id('61da1015-291f-43d1-a1a8-345cff12d201')
    def test_cannot_create_wildcard_NS_recordset(self):
        model = dns_data_utils.wildcard_ns_recordset(self.zone['name'])
        self.assertRaisesDns(
            exceptions.BadRequest, 'invalid_object', 400,
            self.recordset_client.create_recordset, self.zone['id'], model
        )

    @decorators.idempotent_id('92f681aa-d953-4d18-b12e-81a9149ccfd9')
    def test_cname_recordsets_cannot_have_more_than_one_record(self):
        post_model = dns_data_utils.rand_cname_recordset(
            zone_name=self.zone['name'])

        post_model['records'] = [
            "a.{0}".format(self.zone['name']),
            "b.{0}".format(self.zone['name']),
        ]

        self.assertRaises(
            exceptions.BadRequest,
            self.recordset_client.create_recordset,
            self.zone['id'], post_model
        )

    @decorators.idempotent_id('22a9544b-2382-4ed2-ba12-4dbaedb8e880')
    def test_cannot_create_TXT_with(self):
        for key, data in INVALID_TXT_DATASET.items():
            LOG.info('Tested INVALID_TXT_DATASET: {}'.format(key))
            post_model = dns_data_utils.rand_txt_recordset(
                self.zone['name'], data['data'])
            self.assertRaisesDns(
                exceptions.BadRequest, 'invalid_object', 400,
                self.recordset_client.create_recordset,
                self.zone['id'], post_model
            )

    @decorators.idempotent_id('03e4f811-0c37-4ce2-8b16-662c824f8f18')
    def test_create_TXT_with(self):
        for key, data in VALID_TXT_DATASET.items():
            LOG.info('Tested VALID_TXT_DATASET: {}'.format(key))
        post_model = dns_data_utils.rand_txt_recordset(
            self.zone['name'], data['data'])
        recordset = self.create_recordset(post_model)

        waiters.wait_for_recordset_status(
            self.recordset_client, self.zone['id'], recordset['id'], 'ACTIVE')

    @decorators.idempotent_id('775b3db5-ec60-4dd7-85d2-f05a9c544978')
    def test_create_SPF_with(self):
        for key, data in VALID_TXT_DATASET.items():
            LOG.info('Tested VALID_TXT_DATASET: {}'.format(key))
            post_model = dns_data_utils.rand_spf_recordset(
                self.zone['name'], data['data'])
            recordset = self.create_recordset(post_model)

            waiters.wait_for_recordset_status(
                self.recordset_client, self.zone['id'],
                recordset['id'], 'ACTIVE')

    @decorators.idempotent_id('7fa7783f-1624-4122-bfb2-6cfbf7a5b49b')
    def test_cannot_create_MX_with(self):
        for key, pref in INVALID_MX_DATASET.items():
            LOG.info('Tested INVALID_MX_DATASET: {}'.format(key))

            post_model = dns_data_utils.rand_mx_recordset(
                self.zone['name'], pref=pref['pref']
            )

            self.assertRaisesDns(
                exceptions.BadRequest, 'invalid_object', 400,
                self.recordset_client.create_recordset,
                self.zone['id'], post_model,
            )

    @decorators.idempotent_id('3016f998-4e4a-4712-b15a-4e8dfbc5a60b')
    def test_cannot_create_SSHFP_with(self):
        for key, data in INVALID_SSHFP_DATASET.items():
            LOG.info('Tested INVALID_SSHFP_DATASET: {}'.format(key))

            post_model = dns_data_utils.rand_sshfp_recordset(
                zone_name=self.zone['name'],
                algorithm_number=data['algo'],
                fingerprint_type=data['finger'],
            )

            self.assertRaisesDns(
                exceptions.BadRequest, 'invalid_object', 400,
                self.recordset_client.create_recordset,
                self.zone['id'], post_model,
            )

    @decorators.idempotent_id('193be0fb-ac25-44a3-ae12-f7776048c31a')
    def test_create_SVCB_with(self):
        if not versionutils.is_compatible('2.2', self.api_version,
                                          same_major=False):
            raise self.skipException(
                'SVCB record tests require Designate API version 2.2 or '
                'newer. Skipping test_create_SVCB_with test.')

        ipv4hint = "1.2.3.4,9.8.7.6"
        alpn = "h3,h2,http/1.1"
        port = "888"
        target = f"sample.{self.zone['name']}"
        doh = '/dns-query{?dns}'
        svcb_data_records = [f"1 {target} alpn={alpn} ipv4hint={ipv4hint}"
                             f" port={port} dohpath={doh}"]
        recordset_data = {
            'name': "svcb" + "." + self.zone['name'],
            'type': "SVCB",
            'records': svcb_data_records,
        }
        recordset = self.create_recordset(recordset_data)
        waiters.wait_for_recordset_status(
            self.recordset_client, self.zone['id'],
            recordset['id'], 'ACTIVE')

    @decorators.idempotent_id('758c4367-88a6-4657-908e-4c0785428cf9')
    def test_create_HTTPS_with(self):
        if not versionutils.is_compatible('2.2', self.api_version,
                                          same_major=False):
            raise self.skipException(
                'HTTPS record tests require Designate API version 2.2 or '
                'newer. Skipping test_create_HTTPS_with test.')

        ipv4hint = "1.2.3.4,9.8.7.6"
        alpn = "h3,h2,http/1.1"
        port = "4443"
        target = f"sample.{self.zone['name']}"
        https_data_records = [f"1 {target} alpn={alpn} ipv4hint={ipv4hint}"
                              f" port={port}"]
        recordset_data = {
            'name': "https" + "." + self.zone['name'],
            'type': "HTTPS",
            'records': https_data_records,
        }
        recordset = self.create_recordset(recordset_data)
        waiters.wait_for_recordset_status(
            self.recordset_client, self.zone['id'],
            recordset['id'], 'ACTIVE')



# Copyright 2016 Hewlett Packard Enterprise Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from tempest.lib.common.utils import data_utils

from designate_tempest_plugin.common import constants as const

from designate_tempest_plugin import data_utils as dns_data_utils
from designate_tempest_plugin.common import waiters
from designate_tempest_plugin.services.dns.v2.json import base


class ZonesClient(base.DnsClientV2Base):
    """API V2 Tempest REST client for Designate API"""

    @base.handle_errors
    def create_zone(self, name=None, email=None, ttl=None, description=None,
                    attributes=None, wait_until=False,
                    zone_type=const.PRIMARY_ZONE_TYPE,
                    primaries=None, params=None, project_id=None):

        """Create a zone with the specified parameters.

        :param name: The name of the zone.
            Default: Random Value
        :param email: The email for the zone.
            Default: Random Value
        :param ttl: The ttl for the zone.
            Default: Random Value
        :param description: A description of the zone.
            Default: Random Value
        :param attributes: Key:Value pairs of information about this zone,
               and the pool the user would like to place the zone in.
               This information can be used by the scheduler to place
               zones on the correct pool.
        :param wait_until: Block until the zone reaches the desiered status
        :param zone_type: PRIMARY or SECONDARY
            Default: PRIMARY
        :param primaries: List of Primary nameservers. Required for SECONDARY
            Default: None
        :param params: A Python dict that represents the query parameters to
                       include in the request URI.
        :param project_id: When specified, overrides the project ID the zone
                           will be associated with.
        :return: A tuple with the server response and the created zone.
        """

        zone = {
            'name': name or dns_data_utils.rand_zone_name()
            if name != '' else '',
            'email': email or dns_data_utils.rand_email()
            if email != '' else '',
            'ttl': ttl or dns_data_utils.rand_ttl()
            if ttl != 0 else 0,
            'description': description or data_utils.rand_name('test-zone')
            if description != '' else '',
            'attributes': attributes or {
                'attribute_key': data_utils.rand_name('attribute_value')}
        }
        # If SECONDARY, "email" and "ttl" cannot be supplied
        if zone_type == const.SECONDARY_ZONE_TYPE:
            zone['type'] = zone_type
            del zone['email']
            del zone['ttl']
            if primaries is None:
                raise AttributeError(
                    'Error - "primaries" is mandatory parameter'
                    ' for a SECONDARY zone type')

            zone['masters'] = primaries

        headers = None
        extra_headers = False
        if project_id:
            headers = {'x-auth-sudo-project-id': project_id}
            extra_headers = True

        resp, body = self._create_request('zones', zone, params=params,
                                          headers=headers,
                                          extra_headers=extra_headers)

        # Create Zone should Return a HTTP 202
        self.expected_success(202, resp.status)

        if wait_until:
            waiters.wait_for_zone_status(self, body['id'], wait_until,
                                         headers=headers)

        return resp, body

    @base.handle_errors
    def show_zone(self, uuid, params=None, headers=None):
        """Gets a specific zone.
        :param uuid: Unique identifier of the zone in UUID format.
        :param params: A Python dict that represents the query parameters to
                       include in the request URI.
        :param headers (dict): The headers to use for the request.
        :return: Serialized zone as a dictionary.
        """
        return self._show_request(
            'zones', uuid, params=params, headers=headers)

    @base.handle_errors
    def show_zone_nameservers(self, zone_uuid, params=None, headers=None):
        """Gets list of Zone Name Servers
        :param zone_uuid: Unique identifier of the zone in UUID format.
        :param params: A Python dict that represents the query parameters to
                       include in the request URI.
        :param headers (dict): The headers to use for the request.
        :return: Serialized nameservers as a list.
        """
        return self._show_request(
            'zones/{0}/nameservers'.format(zone_uuid), uuid=None,
            params=params, headers=headers)

    @base.handle_errors
    def list_zones(self, params=None, headers=None):
        """Gets a list of zones.
        :param params: A Python dict that represents the query parameters to
                       include in the request URI.
        :param headers (dict): The headers to use for the request.
        :return: Serialized zones as a list.
        """
        return self._list_request('zones', params=params, headers=headers)

    @base.handle_errors
    def delete_zone(self, uuid, params=None, headers=None, delete_shares=None):
        """Deletes a zone having the specified UUID.
        :param uuid: The unique identifier of the zone.
        :param params: A Python dict that represents the query parameters to
                       include in the request URI.
        :param headers (dict): The headers to use for the request.
        :param delete_shares: if set, delete-shares modifier will be activated
        :return: A tuple with the server response and the response body.
        """
        if delete_shares:
            if headers:
                headers['x-designate-delete-shares'] = True
            else:
                headers = {'x-designate-delete-shares': True}
        resp, body = self._delete_request(
            'zones', uuid, params=params, headers=headers)

        # Delete Zone should Return a HTTP 202
        self.expected_success(202, resp.status)

        return resp, body

    @base.handle_errors
    def update_zone(self, uuid, email=None, ttl=None,
                    description=None, wait_until=False, params=None,
                    headers=None):
        """Update a zone with the specified parameters.
        :param uuid: The unique identifier of the zone.
        :param email: The email for the zone.
            Default: Random Value
        :param ttl: The ttl for the zone.
            Default: Random Value
        :param description: A description of the zone.
            Default: Random Value
        :param wait_until: Block until the zone reaches the desiered status
        :param params: A Python dict that represents the query parameters to
                       include in the request URI.
        :param headers (dict): The headers to use for the request.
        :return: A tuple with the server response and the updated zone.
        """
        zone = {
            'email': email or dns_data_utils.rand_email(),
            'ttl': ttl or dns_data_utils.rand_ttl(),
            'description': description or data_utils.rand_name('test-zone'),
        }

        resp, body = self._update_request('zones', uuid, zone, params=params,
                                          headers=headers)

        # Update Zone should Return a HTTP 202
        self.expected_success(202, resp.status)

        if wait_until:
            waiters.wait_for_zone_status(self, body['id'], wait_until)

        return resp, body

    @base.handle_errors
    def trigger_manual_update(self, zone_id, headers=None):
        """Trigger manually update for secondary zone.

        :param zone_id: Secondary zone ID.
        :param headers (dict): The headers to use for the request.
        :return: A tuple with the server response and body.
        """
        resp, body = self._create_request(
            'zones/{}/tasks/xfr'.format(zone_id), headers=headers)
        # Trigger Zone Update should Return a HTTP 202
        self.expected_success(202, resp.status)
        return resp, body

    @base.handle_errors
    def abandon_zone(self, zone_id, headers=None):
        """This removes a zone from the designate database without removing
         it from the backends.

        :param zone_id: Zone ID.
        :param headers (dict): The headers to use for the request.
        :return: A tuple with the server response and body.
        """
        resp, body = self._create_request(
            'zones/{}/tasks/abandon'.format(zone_id),
            headers=headers,
            expected_statuses=self.DELETE_STATUS_CODES)

        self.expected_success(self.DELETE_STATUS_CODES, resp.status)
        return resp, body



# Copyright 2021 Red Hat.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Designate statuses/actions strings
PENDING = 'PENDING'
COMPLETE = 'COMPLETE'
ERROR = 'ERROR'
DELETED = 'DELETED'
DELETE = 'DELETE'
ACTIVE = 'ACTIVE'
INACTIVE = 'INACTIVE'
UP = 'UP'
CREATE = 'CREATE'
UPDATE = 'UPDATE'
NONE = 'NONE'

# Zone types
PRIMARY_ZONE_TYPE = 'PRIMARY'
SECONDARY_ZONE_TYPE = 'SECONDARY'

# DNS Record types
A = 'A'
AAAA = 'AAAA'
ALIAS = 'ALIAS'
CNAME = 'CNAME'
MX = 'MX'
NS = 'NS'
PTR = 'PTR'
SOA = 'SOA'
SRV = 'SRV'
TXT = 'TXT'



# Copyright 2021 Red Hat.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from designate_tempest_plugin.services.dns.v2.json import base


class ServiceClient(base.DnsClientV2Base):

    @base.handle_errors
    def list_statuses(self, headers=None):
        """List all Services and statuses

        :param headers: (dict): The headers to use for the request.
        :return: Serialized service statuses as a list.
        """
        return self._list_request(
            'service_statuses', headers=headers)[1]['service_statuses']

    @base.handle_errors
    def show_statuses(self, uuid, headers=None):
        """Show Service status

        :param headers: (dict): The headers to use for the request.
        :param uuid: service ID
        :return: Service status dictionary
        """
        return self._show_request(
            'service_statuses', uuid, headers=headers)[1]



# Copyright 2021 Red Hat.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from designate_tempest_plugin.services.dns.v2.json import base


class ApiVersionClient(base.DnsClientV2Base):

    @base.handle_errors
    def list_enabled_api_versions(self):
        """Show all enabled API versions

        :return: Dictionary containing version details
        """
        resp, body = self.get('/')
        self.expected_success(self.LIST_STATUS_CODES, resp.status)
        return resp, self.deserialize(resp, body)



"""
Copyright 2015 Rackspace

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


class ZoneFile(object):

    def __init__(self, origin, ttl, records):
        self.origin = origin
        self.ttl = ttl
        self.records = records

    def __str__(self):
        return str(self.__dict__)

    def __repr__(self):
        return str(self)

    def __eq__(self, other):
        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self.__eq__(other)

    @classmethod
    def from_text(cls, text):
        """Return a ZoneFile from a string containing the zone file contents"""
        # filter out empty lines and strip all leading/trailing whitespace.
        # this assumes no multiline records
        lines = [x.strip() for x in text.split('\n') if x.strip()]

        assert lines[0].startswith('$ORIGIN')
        assert lines[1].startswith('$TTL')

        return ZoneFile(
            origin=lines[0].split(' ')[1],
            ttl=int(lines[1].split(' ')[1]),
            records=[ZoneFileRecord.from_text(x) for x in lines[2:]],
        )


class ZoneFileRecord(object):

    def __init__(self, name, type, data):
        self.name = str(name)
        self.type = str(type)
        self.data = str(data)

    def __str__(self):
        return str(self.__dict__)

    def __repr__(self):
        return str(self)

    def __eq__(self, other):
        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self.__eq__(other)

    def __hash__(self):
        return hash(tuple(sorted(self.__dict__.items())))

    @classmethod
    def from_text(cls, text):
        """Create a ZoneFileRecord from a line of text of a zone file, like:

            mydomain.com. IN NS ns1.example.com.
        """
        # assumes records don't have a TTL between the name and the class.
        # assumes no parentheses in the record, all on a single line.
        parts = [x for x in text.split(' ', 4) if x.strip()]
        name, rclass, rtype, data = parts
        assert rclass == 'IN'
        return cls(name=name, type=rtype, data=data)



# Copyright 2021 Red Hat.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_log import log as logging
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


from designate_tempest_plugin.tests import base

CONF = config.CONF
LOG = logging.getLogger(__name__)


class DesignateLimit(base.BaseDnsV2Test):
    credentials = ["admin", "primary", "alt",
                   "project_member", "project_reader"]

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(DesignateLimit, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(DesignateLimit, cls).setup_clients()
        cls.admin_client = cls.os_admin.dns_v2.DesignateLimitClient()
        cls.primary_client = cls.os_primary.dns_v2.DesignateLimitClient()
        cls.alt_client = cls.os_alt.dns_v2.DesignateLimitClient()

    @decorators.idempotent_id('828572be-8662-11eb-8ff2-74e5f9e2a801')
    def test_list_designate_limits_as_primary_user(self):
        expected_default_limits_fields = [
            "max_page_limit", "max_recordset_name_length",
            "max_recordset_records", "max_zone_name_length",
            "max_zone_records", "max_zone_recordsets",
            "max_zones", "min_ttl"].sort()
        project_limits = self.primary_client.list_designate_limits()
        LOG.info(
            'Retrieved designate limits are: {} '.format(project_limits))
        self.assertEqual(
            expected_default_limits_fields,
            list(project_limits.keys()).sort(),
            'Retrieved fields: {} are not as expected: {} '.format(
                list(project_limits.keys()).sort(),
                expected_default_limits_fields))

    @decorators.idempotent_id('828572be-8662-11eb-8ff2-74e5f9e2a801')
    def test_list_designate_impersonate_another_user_as_admin(self):
        primary_project_limits = self.primary_client.list_designate_limits()
        LOG.info(
            'Retrieved designate limits for Primary user are: {} '.format(
                primary_project_limits))
        admin_sudo_project_limits = self.admin_client.list_designate_limits(
            headers={'x-auth-sudo-project-id': self.primary_client.project_id})
        LOG.info(
            'Retrieved designate limits for Admin user impersonates '
            'Primary user are: {} '.format(admin_sudo_project_limits))
        self.assertEqual(
            primary_project_limits, admin_sudo_project_limits,
            'Failed, Admin user should receive the same values for '
            'Designate limits as a Primary tenant did.')

    @decorators.idempotent_id('5975fee0-d430-11eb-aa4d-74e5f9e2a801')
    def test_list_designate_impersonate_another_user_as_alt(self):
        self.assertRaises(
            lib_exc.Forbidden, self.alt_client.list_designate_limits,
            headers={'x-auth-sudo-project-id': self.primary_client.project_id})

    @decorators.idempotent_id('828572be-8662-11eb-8ff2-74e5f9e2a801')
    @decorators.skip_because(bug="1933444")
    def test_list_designate_limits_all_projects(self):
        existing_project_ids = [
            self.primary_client.project_id, self.alt_client.project_id]
        LOG.info('Project IDs we would expect to receive with Admin user '
                 'uses: "x-auth-all-projects" HTTP header '
                 'are {}: '.format(existing_project_ids))
        all_project_limits = self.admin_client.list_designate_limits(
            headers={'x-auth-all-projects': True})
        LOG.info('Retrieved designate limits by Admin user for all projects '
                 'are: %s', all_project_limits)
        received_project_ids = [
            item['project_id'] for item in all_project_limits]
        for project_id in existing_project_ids:
            self.assertIn(
                project_id, received_project_ids,
                'Failed, expected project_id:{} is missing in:{} '.format(
                    project_id, received_project_ids))

    @decorators.idempotent_id('fc57fa6b-5280-4186-9be9-ff4da0961db0')
    def test_list_designate_limits_RBAC(self):
        expected_allowed = ['os_admin', 'os_primary', 'os_alt']
        if CONF.dns_feature_enabled.enforce_new_defaults:
            expected_allowed.extend(['os_project_member', 'os_project_reader'])

        self.check_list_show_RBAC_enforcement(
            'DesignateLimitClient', 'list_designate_limits',
            expected_allowed, False)
